\ ROM PowerForth MSP430 code definitions for Launchpad MSP430G2553

((
Copyright (c) 2002, 2003, 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140211 SFP006 Corrected LSHIFT and RSHIFT.
20140121 SFP005 Overhauled defining words for Flash.
20120825 SFP004 Converted to LITE form
20120824 SFP003 Overhauled branch code.
		Added BIGKERNEL? switch.
20030825 SFP002 Forced alignment in (").
20030605 SFP001 Improved D>
))

\ **********
\ *! code430lite
\ *T MSP430 code definitions
\ **********
\ *P The file *\i{MSP430\Code430lite.fth} contains all the code
\ ** definitions needed for a small standalone Forth kernel. Such a
\ ** kernel is practical for the MSP430G2553 that has enough Flash
\ ** and RAM. See *\i{liteLP2553sa.ctl} for the control file.

\ *P If you are using Umbilical Forth, treat *\i{MSP430\Code430lite.fth}
\ ** as a source repository from which you can copy required words.

\ *****************
\ *S Register usage
\ *****************
\ *P On the MSP430 the following register usage is the default:
\ *E Forth        MSP430        Comments
\ ** -----------------------------------
\ ** IP           R0/PC         MSP430 PC
\ ** RSP          R1/SP         MSP430 return stack
\ ** --           R2/CG1/SR
\ ** --           R3/CG2
\ ** PSP          R4            data stack pointer
\ ** TOS          R5            cached top of data stack
\ ** UP           R6            USER area pointer
\ ** LP (locals)  R7            points to LOCALs on r. stack
\ ** scratch      R8..R13       with R5 forms working stack
\ ** codegen      R14           code generator temporary
\ ** codegen      R15           preserves SR in shuffle
\ ** ------------------------------------

\ *P The VFX code generator reserves R14 and R15 for internal
\ ** operations. *\fo{CODE} definitions must use R5 as TOS with
\ ** NOS pointed to by R4 as a descending stack. R8..R15 are
\ ** free for use by CODE definitions and need not be preserved or
\ ** restored. You should assume that any register can be affected
\ ** by other words.


\ ***************************
\ *S Literal and flow of control
\ ***************************

((
code lit	\ -- x ; remove when target LITERAL is optimized
 \ *G Code which when CALLED at runtime will return an inline cell
 \ ** value. Remove when target LITERAL is optimized.
  sub  # 2 psp			\ save TOS
  mov  tos  0 (psp)
  pop  r14			\ return address
  mov  @r14+ tos		\ inline data
  br   r14
end-code
))

code execute    \ xt --
\ *G Execute the code described by the XT. This is a Forth equivalent
\ ** to an assembler JSR/CALL instruction.
  mov  tos r15
  mov  @psp+ tos		\ pop TOS
  br   r15			\ go do it
end-code

proc docreate	\ -- a-addr
\ *G The run time action of *\fo{CREATE}.
  sub  # 2 psp			\ save TOS
  mov  tos  0 (psp)
  pop  r14			\ return address
  mov  @r14  tos		\ inline address
  ret
end-code

internal

CODE (DO)       \ limit start --
\ *G The run time action of *\fo{DO} compiled on the target.
l: pdo1
  pop   r15			\ return address
  push  @r15+			\ push loop exit address
  mov   @psp+  r14		\ push limit xor $8000
  xor   # $8000  r14
  push  r14
  sub   r14 tos			\ push start-(limit xor $8000)
  push  tos
  mov   @psp+ tos		\ restore tos
  br    r15
end-code

CODE (?DO)      \ limit start --
\ *G The run time action of *\fo{?DO} compiled on the target.
  cmp  @psp tos			\ ok if not same
  jne  pdo1
  add  # 2 psp			\ remove limit and start
  mov  @psp+ tos
  pop  r15			\ return address contains loop exit address
  br   @r15
end-code

CODE (LOOP)     \ -- ; absolute address follows inline
\ *G The run time action of *\fo{LOOP} compiled on the target.
  pop  r15			\ return address
  inc  0 (rsp)			\ update
  bit  # $0100 sr		\ test overflow bit
  jnz  ploop1			\ done if set
  br   0 (r15)			\ go round again
l: ploop1
  incd r15			\ step over branch address
  add  # 6 rsp			\ remove loop parameters
  br   r15			\ return
end-code

CODE (+LOOP)    \ n --
\ *G The run time action of *\fo{+LOOP} compiled on the target.
  pop  r15			\ return address
  add  tos  0 (rsp)		\ update
  mov  @psp+ tos		\ restore tos
  bit  # $0100 sr		\ test overflow bit
  jnz  p+loop1			\ done if set
\  mov  @psp+ tos		\ restore tos
  br   0 (r15)			\ go round again
l: p+loop1
  incd r15			\ step over branch address
  add  # 6 rsp			\ remove loop parameters
\  mov  @psp+ tos		\ restore tos
  br   r15			\ return
end-code

external

code i          \ -- n ; return DO ... LOOP index
\ *G Return the current index of the inner-most *\fo{DO..LOOP}.
  sub  # 2 psp
  mov  tos 0 (psp)
  mov  2 (rsp) tos
  add  4 (rsp) tos
  ret
end-code

((
code j		\ -- n ; return outer DO ... LOOP index
\ *G Return the current index of the outer *\fo{DO..LOOP}.
  sub  # 2 psp
  mov  tos 0 (psp)
  mov  8 (rsp) tos
  add  #10 (rsp) tos
  ret
end-code
))

code unloop     \ -- ; discard DO ... LOOP parameters ; ANS 6.1.2380
\ *G Remove the *\fo{DO..LOOP} control parameters from the return stack.
  pop   r14
  add   # 6 rsp
  br    r14
end-code

CODE LEAVE      \ -- ; leave DO ... LOOP
\ *G Remove the current *\fo{DO..LOOP} parameters and jump to the
\ ** end of the *\fo{DO..LOOP} structure.
  add   # 6 rsp
  ret
end-code

((
code ?leave	\ flag -- ; leave DO ... LOOP
 \ *G If *\i{flag} is non-zero, remove the current *\fo{DO..LOOP}
 \ ** parameters and jump to the end of the *\fo{DO..LOOP} structure.
  cmp	# 0  tos		\ test flag
  mov   @psp+  tos		\ restore TOS, does not affect flags
  jz	qleavex
  add	# 6  rsp
l: qleavex
  ret
end-code
))


\ *******************
\ *S Flash operations
\ *******************

internal

code (c!f)	\ w addr --
\ +G 16 bit Flash write primitive.
  mov	@psp+  r15		\ r15 = w
  mov .b  @tos  r14		\ check already done
  cmp .b  r14  r15
  jz    pc!fx
  push	sr			\ disable interrupts
  dint
  mov   # $A500  & FCTL3	\ clear all bits, especially LOCK bit
  mov   # $A540  & FCTL1	\ set WRT to enable write
  mov .b   r15  0 (tos)		\ write
  mov   # $A500  & FCTL1	\ clear WRT to disable write
  mov   # $A510  & FCTL3	\ set LOCK bit
  pop	sr			\ restore interrupts
l: pc!fx
  mov	@psp+  tos		\ restore TOS
  ret
end-code

code (!f)	\ w addr --
\ +G 16 bit Flash write primitive.
  mov	@psp+  r15		\ r15 = w
  bit	# 1  tos		\ test for even address
  jnz   p!fx
  mov   @tos  r14		\ check already done
  cmp	r14  r15
  jz    p!fx
  push	sr			\ disable interrupts
  dint
  mov   # $A500  & FCTL3	\ clear all bits, especially LOCK bit
  mov   # $A540  & FCTL1	\ set WRT to enable write
  mov   r15  0 (tos)		\ write
  mov   # $A500  & FCTL1	\ clear WRT to disable write
  mov   # $A510  & FCTL3	\ set LOCK bit
  pop	sr			\ restore interrupts
l: p!fx
  mov	@psp+  tos		\ restore TOS
  ret
end-code

code (fe)	\ addr --
\ +G Erase flash sector at the given address.
\ Segment Erase from flash.
\ Assumes ACCVIE = NMIIE = OFIE = 0, watchdog disabled.
\ Per section 5.3.2 of MSP430 Family User's Guide
  push	sr			\ disable interrupts
  dint
  mov   # $A500  & FCTL3	\ clear all bits, especially LOCK bit
  mov   # $A502  & FCTL1	\ set ERASE to enable write
  mov   # -1  0 (tos)		\ write
  mov   # $A500  & FCTL1	\ clear ERASE to disable write
  mov   # $A510  & FCTL3	\ set LOCK bit
  pop	sr			\ restore interrupts
  mov	@psp+  tos		\ restore TOS
  ret
end-code

: inFlash?	\ addr -- flag
\ +G Returns true if the address is in the info or main Flash areas.
  dup INFOSTART INFOEND within		\ Info Flash
  swap FLASHSTART u< 0= or		\ Main flash xxxx..FFFF
;

external

: FlErase	\ addr len --
\ *G Erase the Flash sectors in the given range.
  bounds ?do
    i FlashStart u> if
      i (fe) #512
    else
      i INFOSTART INFOEND 1+ within if
        i (fe) #128
      else
        i (fe) 2
      endif
    endif
  +loop
;

: c!f		\ b addr --
\ *G Program any address, including Flash, with an 8 bit value.
  dup InFlash?
  if  (c!f)  else c!  endif
;

: !f		\ w addr --
\ *G Program any address, including Flash, with a 16 bit value.
  dup InFlash?
  if  (!f)  else  !  endif
;

internal
: >Info		\ src dest len --
\ +G Save a memory block to Info Flash. The src address must be
\ +* even. The destination address must be the start address of
\ +* an Info Flash sector. The kernel CRC is added at the end.
  2dup FlErase
  2dup + crcslot @ swap !f		\ save kernel checksum
  bounds ?do
    dup @ i !f  cell +
  cell +loop
  drop
;
external


\ *********************
\ +S Structure compilation
\ *********************
\ +P These words define high level branches. They are used by the structure
\ +* words such as *\fo{IF} and *\fo{AGAIN}.


internal

\ ----------------------------
\ +N High level branch resolution
\ ----------------------------

: >mark         \ -- addr ; mark start of forward branch
\ +G Mark the start of a forward branch. HIGH LEVEL CONSTRUCTS ONLY.
  here  ( 0 , ) cell allot
;

: >resolve      \ addr -- ; resolve absolute target of forward branch
\ +G Resolve absolute target of forward branch. HIGH LEVEL CONSTRUCTS ONLY.
  here swap !f  ;                        \ absolute

compiler
: <mark         \ -- addr ; mark start (destination) of backward branch
\ +G Mark the start (destination) of a backward branch.
\ +* HIGH LEVEL CONSTRUCTS ONLY.
  here
;

: <resolve      \ addr ; resolve (at branch point) backward branch
\ +G Mark the start (destination) of a backward branch.
\ +* HIGH LEVEL CONSTRUCTS ONLY.
   ,                    		\ absolute for target
;
target


\ ---------------------------
\ +N Low level branch resolution
\ ---------------------------

: >off10	\ dest orig -- off10
\ +G Calculate an MSP430 10 bit offset for a branch instruction.
  2 + -  1 rshift $03FF and
;

: >res10	\ orig --
\ +G Resolve a 10 bit forward branch opcode at *\i{orig}.
  here over >off10			\ generate offset
  over @  $FC00 and or  swap !f		\ get opcode, merge, put back
;

: <res10,	\ dest opcode --
\ +G Lay *\i{opcode} with a 10 bit branch to *\i{dest}.
  swap  here >off10  or ,		\ offset, merge and lay opcode
;

compiler
: >c_res_branch	\ orig -- ; fix up forward referenced branch
\ +G See *\fo{>RES10}.
  >res10
;
: c_mrk_branch<	\ -- addr ; mark destination of backward branch
\ +G See *\fo{HERE}.
  here  ;
target


\ -------------------
\ Branch constructors
\ -------------------

: c_branch<     \ dest --
\ +G Lay the code for an unconditional branch to *\i{dest}.
  $3C00  <res10,			\ jmp addr
;

: c_?branch<    \ dest -- ; lay ?BRANCH instructions
\ +G Lay the code for a conditional branch to *\i{dest}
  $9305 ,				\ cmp  # 0 r5
  $4435 ,				\ mov  @r4+ r5
  $2400 <res10,				\ jeq  addr
;

: c_branch>     \ -- addr ; forward referenced branch
\ +G Lay the code for a forward referenced unconditional branch.
   here  $3C00 $03FF or ,
;

: c_?branch>    \ -- addr ; forward referenced branch
\ +G Lay the code for a forward referenced conditional branch.
  $9305 ,				\ cmp  # 0 r5
  $4435 ,				\ mov  @r4+ r5
  here  $2400 $03FF or ,		\ jeq  $+2
;


\ ------------------------
\ Main structure compilers
\ ------------------------

: c_lit		\ lit --
\ +G Compile the code for a LITERAL.
  $8324 ,				\ SUB  # 2 R4
  $4584 ,  0 ,				\ MOV  R5 0000 (R4)
  $4035 ,  ,				\ MOV  # lit  R5
;

: c_exit        \ --
\ +G Compile the code for *\fo{EXIT}.
  $4130 ,				\ RET/NEXT
;

: c_do          \ C: -- do-sys ; Run: n1|u1 n2|u2 -- ; R: -- loop-sys
\ +G Compile the code for *\fo{DO}.
  compile (do) >mark <mark
;

: c_?DO         \ C: -- do-sys ; Run: n1|u1 n2|u2 -- ; R: -- | loop-sys
\ +G Compile the code for *\fo{?DO}.
  compile (?do) >mark <mark
;

: c_LOOP        \ C: do-sys -- ; Run: -- ; R: loop-sys1 -- | loop-sys2
\ +G Compile the code for *\fo{LOOP}.
  compile (loop) <resolve >resolve
;

: c_+LOOP       \ C: do-sys -- ; Run: -- ; R: loop-sys1 -- | loop-sys2
\ +G Compile the code for *\fo{+LOOP}.
  compile (+loop) <resolve >resolve
;

external


\ *********************
\ *S Digits and strings
\ *********************

code DIGIT         \ char base -- 0 | n true
\ *G If the ASCII value *\i{char} can be treated as a digit for a
\ ** number in the given *\i{base} then return the digit and a
\ ** TRUE flag, otherwise just return FALSE. For bases greater
\ ** than 10, the letters A..Z are used, e.g. 0..9,A..F for
\ ** hexadecimal.
  mov	@psp+ r15		\ r15 = char, tos = base
  sub	# $30  r15
  jn    dig2			\ fail if char < '0'
  cmp	# $0A  r15
  jl	dig1			\ true, '0' <= char <= '9'
  sub	# 7  r15		\ convert 'A'..'F'
  cmp	# $0A  r15
  jl	dig2			\ fail if result < 'A'
l: dig1
  cmp	tos  r15		\ range check of base
  jge   dig2
  sub	# 2  psp		\ return digit
  mov	r15  0 (psp)
  mov	# -1 tos		\ return true
  ret
l: dig2
  mov	# 0  tos
  ret
end-code

: /string    \ addr len n -- addr+n len-n
\ *G Modify a string address and length to remove the first N characters
\ ** from the string.
  tuck 2swap  + -rot  -
;

CODE CMOVE      \ source dest len -- ; copy memory areas
\ *G Copy U bytes of memory forwards from C-ADDR1 to C-ADDR2.
  mov  @psp+ r13		\ r13=dest
  mov  @psp+ r12		\ r12=src
  cmp  # 0 tos
  nz, if,
    begin,
      mov .b  @r12+ 0 (r13)
      inc     r13
      dec     tos
    z, until,
  endif,
  mov  @psp+ tos
  ret
end-code

CODE CMOVE>      \ source dest len -- ; copy memory areas
\ *G As *\fo{CMOVE} but working in the opposite direction,
\ ** copying the last character in the string first.
  mov  @psp+ r13		\ r13=dest
  mov  @psp+ r12		\ r12=src
  cmp  # 0 tos
  nz, if,
    add  tos r12
    add  tos r13
    begin,
      dec     r12
      dec     r13
      mov .b  @r12  0 (r13)
      dec     tos
    z, until,
  endif,
  mov  @psp+ tos
  ret
end-code

code fill	\ addr len char --
\ *G Fill *\i{len} bytes of memory starting at *\i{addr} with the
\ ** byte specified as *\i{char}.
  mov    @psp+  r15			\ r15 := len
  mov    @psp+  r14			\ r14 := addr
  cmp    # 0 r15
  ne, if,
    begin,
      mov .b tos  0 (r14)
      inc    r14
      dec    r15
    z, until,
  endif,
  mov  @psp+ tos
  ret
end-code

: ERASE         \ addr len --
\ *G Fill *\i{len} bytes of memory starting at *\i{addr} with zero.
  0 fill
;

code s=         \ addr1 addr2 count -- flag
\ *G Compare two same-length strings/memory blocks, returning TRUE if
\ ** they are identical.
  mov    @psp+  r15			\ r15 := dest
  mov    @psp+  r14			\ r14 := src
  cmp    # 0 tos
  jeq    s=fail
l: s=loop
  cmp .b @r14+  0 (r15)
  jne    s=fail
  inc    r15
  sub    # 1 tos
  jne    s=loop
  mov    # -1 tos
  ret
l: s=fail
  sub    tos tos
  ret
end-code

code SKIP	\ c-addr u char -- 'c-addr 'u
\ *G Modify the string description by skipping over leading occurrences of
\ ** 'char'.
	mov	@psp+  r15		\ R15 = u
	mov	@psp  r14		\ R14 = caddr
	cmp	# 0  r15
	jeq	skipx
l: skiplp
	cmp .b	@r14  tos		\ match?
	jnz	skipx
	add	# 1 r14			\ step on
	sub	# 1 r15			\ count down
	jnz	skiplp
l: skipx
	mov	r15  tos		\ tos = len'
	mov	r14  0 (psp)		\ nos = caddr'
	ret
end-code


code scan	\ caddr u char -- caddr2 u2
\ *G Look for first occurrence of CHAR in string and return new string.
\ ** C-addr2/u2 describe the string with CHAR as the first character.
	mov	@psp+  r15		\ R15 = u
	mov	@psp  r14		\ R14 = caddr
	cmp	# 0  r15
	jeq	scanx
l: scanlp
	cmp .b	@r14  tos		\ match?
	jz	scanx
	add	# 1 r14			\ step on
	sub	# 1 r15			\ count down
	jnz	scanlp
l: scanx
	mov	r15  tos		\ tos = len'
	mov	r14  0 (psp)		\ nos = caddr'
	ret
end-code

: count		\ addr -- addr+1 len
\ *G Given the address of a counted string in memory this word will
\ ** return the address of the first character and the length in
\ ** characters of the string.
  count  ;

: (")           \ -- addr ; in-line string follows caller, and skip it
\ *G Return the address of a counted string that is inline after the
\ ** CALLING word, and adjust the CALLING word's return address to
\ ** step over the inline string. See the definition of (.") for
\ ** an example.
  r>					\ return address to caller
  r> dup count + 1+ -2 and >r		\ caller's return address
  swap >r                               \ restore return address
;

: upc		\ char -- char' ; force upper case
\ *G Convert char to upper case.
  dup [char] a >= if
    dup [char] z <= if
      $DF and
    endif
  endif
;

: upper		\ c-addr len --
\ *G Convert the ASCII string described to upper-case. This operation
\ ** happens in place.
  bounds ?do  i c@ upc i c!  loop
;

: PLACE         \ c-addr1 u c-addr2 --
\ *G Place the string c-addr1/u as a counted string at c-addr2.
  2dup 2>r  1+ swap move  2r> c!
;


\ *************
\ *S Arithmetic
\ *************

\ ------
\ *N Basics
\ ------

((
: 1+ 1+ ;       \ n -- n+1
\ *G Add one to top-of stack.

: 2+ 2+ ;       \ n -- n+2
\ *G Add two to top-of stack.

: 1- 1- ;       \ n -- n-1
\ *G Subtract one from top-of stack.

: 2- 2- ;       \ n -- n-2
\ *G Subtract two from top-of stack.

: 2* 2* ;       \ n1 -- n2
\ *G Signed multiply top of stack by 2.

: u2/ u2/ ;     \ n1 -- n2 ; unsigned
\ *G Unsigned divide top of stack by 2.

: 2/ 2/ ;       \ n1 -- n2 ; signed
\ *G Signed divide top of stack by 2.
))

: - - ;         \ n1 n2 -- n1-n2
\ *G Subtract two single precision integer numbers. N3|u3=n1|u1-n2|u2.

: + + ;         \ n1 n2 n1+n2
\ *G Add two single precision integer numbers.

: min		\ n1 n2 -- n1|n2 ; return lesser of n1 and n2
  2dup > if swap then drop
;

: max		\ n2 n1 -- n1|n2 ; return greater of n1 and n2
  2dup < if swap then drop
;

: negate	\ n1 -- -n1
\ *G Negate a single precision integer number.
  negate  ;

((
: ?negate       \ n1 t/f -- n1/-n1
 \ *G If flag is negative, then negate n1.
  0< if negate then
;
))

: abs		\ n1 --  |n1|
\ *G If n is negative, return its positive equivalent (absolute value).
  dup 0< if negate then
;

: dnegate	\ d1 -- -d1
\ *G Negate a double number.
  0,0 2swap d-
;

((
: ?dnegate      \ d1 t/f -- d1/-d1
 \ *G If flag is negative, then negate d1.
  0< if dnegate then ;
))

: dabs		\ d1 -- |d1|
\ *G If d is negative, return its positive equivalent (absolute value).
  dup 0< if dnegate then ;

CODE D+         \ d1 d2 -- d3
\ *G Add two double precision integers.
  add  @psp+  2 (psp)
  addc @psp+  tos
  ret
end-code

CODE D-         \ d1 d2 -- d1-d2
\ *G Subtract two double precision integers. D3=D1-D2.
  sub  @psp+ 2 (psp)
  subc tos   0 (psp)
  mov  @psp+  tos
  ret
end-code

CODE S>D        \ n -- d
\ *G Convert a single number to a double one.
  sub  # 2 psp
  mov  tos  0 (psp)
  swpb tos			\ bit 15 -> bit 7
  sxt  tos
  swpb tos
  sxt tos
  ret
end-code

((
: m+            \ d1|ud1 n -- d2|ud2 ; ANS 8.6.1.1830
 \ *G Add double d1 to sign extended single n to form double d2.
  s>d d+  ;
))

: D<            \ d1 d2 -- t/f
\ *G Return TRUE if the double number d1 is < the double number d2.
  d- nip 0<  ;

: d>            \ d1 d2 -- t/f
\ *G Return TRUE if the double number d1 is > the double number d2.
  2swap d- nip 0<  ;

: d0=		\ d -- t/f
\ *G Returns true if d is 0.
  or 0=  ;

: d=         \ d1 d2 -- t/f
\ *G Return TRUE if the two double numbers are equal.
  d- d0=  ;


\ --------------
\ *N Multiplication
\ --------------

proc umul16*16	\ r12:r13 = r14*r15, destroys r11
  mov  # #16  r11		\ counter
  mov  # 0 r12			\ initialise result
  mov  # 0 r13
  begin,
    add  r13 r13  addc r12 r12	\ shift result
    add  r14 r14		\ test r14 top bit
    cy, if,
      add  r15 r13  addc  # 0 r12
    then,
    dec  r11
  z, until,
  ret
end-code

code um*        \ u1 u2 -- ud ; unsigned multiply
\ *G Perform unsigned-multiply between two numbers and return double
\ ** result.
  mov   tos  r14		\ r14=u2
  mov   @psp  r15		\ r15=u1
  call  # umul16*16		\ result in R12:R13
  mov   r12  tos
  mov   r13  0 (psp)
  ret
end-code

code *		\ n1 n2 -- n1*n2 ; signed multiply, : * um* drop  ;
\ *G Standard signed multiply. N3 = n1 * n2.
  mov   tos  r14		\ r14=u2
  mov   @psp+ r15		\ r15=u1
  call  # umul16*16		\ result in R12:R13
  mov   r13  tos
  ret
end-code

code m*		\ n1 n2 -- d ; signed multiply
\ *G Signed multiply yielding double result.
  mov   tos  r14		\ r14=n2
  mov   @psp  r15		\ r15=n1
  mov   r14  r10		\ form sign of result
  xor   r15  r10
  cmp   # 0  r14		\ make n2 positive
  lt, if,  inv  r14   inc  r14  then,
  cmp   # 0  r15		\ make n1 positive
  lt, if,  inv  r15   inc  r15  then,
  call  # umul16*16		\ result in R12:R13
  cmp   # 0 r10			\ test sign of result
  lt, if,
    inv  r12   inv r13		\ negate by invert and add 1
    add  # 1 r13   addc  # 0 r12
  endif,
  mov   r12  tos
  mov   r13  0 (psp)
  ret
end-code


\ --------
\ *N Division
\ --------

proc udiv32/16	\ r12:r13 / r14 ; quot=r13, rem=r12 ; destroys r11 r15
\ +G Division subroutine: performs a 32 bit by 16 bit division
\ +* returning 16 bit quotient and 16 bit remainder.
  cmp  r12 r14			\ check for overflow
  jlo  udiv-oflo
  jeq  udiv-oflo

  clr   r15           \ clear result
  mov   # #17 r11
l: div1
  cmp   r14 r12
  jlo   div2
  sub   r14 r12
l: div2
  rlc   r15
  jc    div4
  dec   r11
  jz    div3
  rla   r13
  rlc   r12
  jnc   div1
  sub   r14 r12
  jmp   div2
l: div3
  clrc
l: div4
  mov   r15 r13
  ret

l: udiv-oflo
  mov  # $FFFF r13		\ quot
  mov  # 0 r12			\ rem
  ret
end-code

code um/mod     \ u32 u16 -- urem uquot
\ *G Perform unsigned division of double number UD by single number U
\ ** and return remainder and quotient.
  mov  tos r14			\ u16
  mov  @psp+ r12		\ u32h
  mov  @psp  r13		\ u32l
  call # udiv32/16		\ quot=r13, rem=r12
  mov  r12 0 (psp)		\ rem
  mov  r13 tos			\ quot
  ret
end-code

code sm/rem      \ d n -- rem quot ; symmetric division
\ *G Perform a signed division of double number D1 by single number N2
\ ** and return remainder and quotient using symmetric (normal) division.
  mov  tos r14			\ n16
  mov  @psp+ r12		\ d32h
  mov  @psp  r13		\ d32l
  mov  r14 r10			\ form sign of quotient
  xor  r12 r10
  mov  r12 r9			\ sign of dividend/remainder
  cmp  # 0 r12			\ make dividend positive
  lt, if,
    inv  r12   inv r13		\ negate by invert and add 1
    add  # 1 r13   addc  # 0 r12
  endif,
  cmp  # 0 r14			\ make divisor positive
  lt, if,
    inv  r14    inc  r14
  endif,
  call # udiv32/16		\ quot=r13, rem=r12
  cmp  # 0 r9			\ remainder takes sign of dividend
  lt, if,  inv  r12  inc  r12  endif,
  cmp  # 0 r10			\ set sign of quotient
  lt, if,  inv  r13  inc  r13  endif,
  mov  r12 0 (psp)		\ rem
  mov  r13 tos			\ quot
  ret
end-code

: /mod          \ n1 n2 -- rem quot
\ *G Signed division of N1 by N2 single-precision yielding remainder
\ ** and quotient.
  >r s>d r> sm/rem
;

: /		\ n1 n2 -- quot
\ *G Standard signed division operator. n3 = n1/n2.
  >r s>d r> sm/rem nip
;

: mod           \ n1 n2 -- rem
\ *G Return remainder of division of N1 by N2. n3 = n1 mod n2.
  /mod drop
;

: MU/MOD        \ ud1 u2 -- u3 ud4
\ *G Perform an unsigned divide of a double ud1 by a single u2, returning
\ ** a single remainder u3 and a double quotient ud4.
  >R 0 R@ UM/MOD R> SWAP >R UM/MOD R>
;


\ *****
\ *S Logic
\ *****

: AND  AND ;      \ n1 n2 -- n3
\ *G Perform a logical AND between the top two stack items and retain
\ ** the result in top of stack.
: OR   OR  ;      \ n1 n2 -- n3
\ *G Perform a logical OR between the top two stack items and retain
\ ** the result in top of stack.
: XOR  XOR ;      \ n1 n2 -- n3
\ *G Perform a logical XOR between the top two stack items and retain
\ ** the result in top of stack.
: INVERT  INVERT ;	\ n1 -- n2
\ *G Perform a bitwise NOT on the top stack item and retain result.


\ ******
\ *S Shifts
\ ******

code lshift	\ x count -- x'
\ *G Left shift x by count bits.
  mov    tos  r14			\ R14 := count
  mov    @psp+  tos			\ TOS := x
  cmp    # 0 r14			\ count != 0 ?
  ne, if,
    begin,
      add    tos  tos
      sub    # 1 r14
    eq, until,
  endif,
  ret
end-code

code rshift	\ x count -- x'
\ *G Right shift x by count bits.
  mov    tos  r14
  mov    @psp+  tos
  cmp    # 0 r14			\ count != 0 ?
  ne, if,
    begin,
      clrc
      rrc    tos
      sub    # 1 r14
    eq, until,
  endif,
  ret
end-code


\ *****************************
\ +S Stack pointer manipulation
\ *****************************

((
  code sp@        \ -- n ; return data stack pointer
  \ *G Get the current address value of the data-stack pointer.
    sub  # 2 psp
    mov  tos  0 (psp)
    mov  psp  tos
    ret
  end-code

  code rp@        \ -- n ; get return stack pointer
  \ *G Get the current address value of the return-stack pointer.
    sub  # 2 psp
    mov  tos  0 (psp)
    mov  rsp  tos
    ret
  end-code

  code sp!        \ n -- ; set data stack pointer
  \ *G Set the current address value of the data-stack pointer.
    mov  tos psp
    mov  @psp+  tos
    ret
  end-code

  code rp!        \ n -- ; set return stack pointer
  \ *G Set the current address value of the return-stack pointer.
    pop  r14
    mov  tos  rsp
    mov  @psp+ tos
    br   r14
  end-code
))


\ ******************
\ *S Return stack words
\ ******************

CODE >R 	\ x -- ; R: -- x
\ *G Push the current top item of the data stack onto the top of the
\ ** return stack.
  pop    r14
  push   tos
  mov    @psp+ tos
  br     r14
end-code

CODE R@ 	\ --  x  ; R:  x -- x
\ *G Copy the top item from the return stack to the data stack.
  sub    # 2 psp
  mov    tos  0 (psp)
  mov    2 (rsp)  tos
  ret
end-code

CODE R> 	\ -- x ; R: x --
\ *G Pop the top item from the return stack to the data stack.
  pop    r14
  sub    # 2 psp
  mov    tos  0 (psp)
  pop    tos
  br     r14
end-code


\ ***********
\ *S Comparisons
\ ***********

: =		\ n1 n2 -- flag
\ *G Return TRUE if the two topmost stack items are equal.
  =  ;

: <>		\ n1 n2 -- flag
\ *G Return TRUE if the two topmost stack items are different.
  <>  ;

: 0<>		\ n -- flag
\ *G Compare the top stack item with 0 and return TRUE if not-equal.
  0<>  ;

: 0=		\ n -- flag
\ *G Compare the top stack item with 0 and return TRUE if equals.
  0=  ;

: 0<		\ n -- flag
\ *G Return TRUE if the top of stack is less-than-zero.
  0<  ;

: 0>		\ n -- flag
\ *G Return TRUE if the top of stack is greater-than-zero.
  0>  ;

: U<		\ n1 n2 -- flag
\ *G An UNSIGNED version of <.
  u<  ;

: U>		\ n1 n2 -- flag
\ *G An UNSIGNED version of >.
  u>  ;

: <		\ n1 n2 -- t/f
\ *G Return TRUE if n1 is less than n2.
  <  ;

: >		\ n1 n2 -- t/f
\ *G Return TRUE if n1 is greater than n2.
  >  ;

: <=		\ n1 n2 -- t/f
\ *G Return TRUE if n1 is less than or equal to n2.
  <=  ;

: >=		\ n1 n2 -- t/f
\ *G Return TRUE if n1 is greater than or equal to n2.
  >=  ;


\ ****************
\ *S Stack primitives
\ ****************

: OVER over ;   \ n1 n2 -- n1 n2 n1
\ *G Copy NOS to a new top-of-stack item.

: 2OVER		\ n1 n2 n3 n4 -- n1 n2 n3 n4 n1 n2
\ *G Similar to *\fo{OVER} but works with cell-pairs rather than cell items.
  2over  ;

: DROP drop ;   \ n1 --
\ *G Lose the top data stack item and promote NOS to TOS.

: 2DROP 2drop ; \ n1 n2 --
\ *G Discard the top two data stack items.

: SWAP swap ;   \ n1 n2 -- n2 n1
\ *G Exchange the top two data stack items.

: 2SWAP		\ n1 n2 n3 n4 -- n3 n4 n1 n2
\ *G Exchange the top two cell-pairs on the data stack.
  2swap  ;

: DUP dup ;     \ n1 -- n1 n1
\ *G *\fo{DUP}licate the top stack item.

: 2DUP 2dup ;   \ n1 n2 -- n1 n2 n1 n2
\ *G *\fo{DUP}licate the top cell-pair on the data stack.

: ?dup		\ n1 -- n1 [n1]
\ *G *\fo{DUP}licate the top stack item only if it non-zero.
  ?dup  ;

: nip nip ;     \ n1 n2 -- n2
\ *G Dispose of the second item on the data stack.

: tuck		\ n1 n2 -- n2 n1 n2
\ *G Insert a copy of the top data stack item underneath the current
\ ** second item.
  tuck  ;

: pick		\ nn..n0 n -- nn..n0 nn
\ *G Get a copy of the Nth data stack item and place on top of stack.
\ ** *\fo{0 PICK} is equivalent to *\fo{DUP}.
  pick  ;

: ROT		\ n1 n2 n3 -- n2 n3 n1
\ *G ROTate the positions of the top three stack items such that the
\ ** current top of stack becomes the second item. See also *\fo{ROLL}.
  rot  ;

: -rot		\ n1 n2 n3 -- n3 n1 n2
\ *G The inverse of *\fo{ROT}.
  -rot  ;


\ ****************
\ Memory operators
\ ****************

: C@ c@ ;       \ addr -- b
\ *G Fetch and 0 extend the character at memory ADDR and return.

: @ @ ;         \ addr -- n
\ *G Fetch and return the CELL at memory ADDR.

: 2@ 2@ ;       \ addr -- d
\ *G Fetch and return the two CELLS from memory ADDR and ADDR+sizeof(CELL).
\ ** The cell at the lower address is on the top of the stack.

: C!		\ b addr --
\ *G Store the character CHAR at memory C-ADDR.
  c!  ;

: !		\ n addr --
\ *G Store the CELL quantity N at memory ADDR.
  !  ;

: 2!		\ d addr --
\ *G Store the two CELLS x1 and x2 at memory ADDR.
\ ** X2 is stored at ADDR and X1 is stored at ADDR+CELL.
  2!  ;

CODE +!         \ n addr --
\ *G Add N to the CELL at memory address ADDR.
  add    @psp+  0 (tos)
  mov    @psp+  tos
  ret
end-code


\ *************
\ Miscellaneous
\ *************

code noop       \ -- ; dummy
\ *G A NOOP, null instruction.                                          )
  ret
end-code

: within 	\ n1|u1 n2|u2 n3|u3 -- flag ; ANS 6.2.2440
\ *G The ANS version of *\fo{WITHIN?}.
\ ** This word uses unsigned arithmetic, so that signed compares are
\ ** treated as existing on a number circle.
  over - -rot - u>
;

: on		\ addr --
\ *G Given the address of a CELL this will set its contents to TRUE (-1).
  on  ;

: off		\ addr --
\ *G Given the address of a CELL this will set its contents to FALSE (0).
  off  ;

: bounds	\ addr len -- addr+len addr
\ *G Convert an address and length to adress+length and address
\ ** as required for *\fo{DO..LOOP} to use *\fo{I} as the current address.
  bounds  ;

compiler
: set-bit	( mask addr -- )  bor!  ;
: reset-bit	( mask addr -- )  bbic!  ;
: toggle-bit	( mask addr -- )  bxor!  ;
: test-bit	( mask addr -- b )  c@ and  ;
target

: name>         \ nfa -- cfa ; convert name address to CFA
\ *G Move a pointer from an NFA to the CFA or "XT" in ANS parlance.
  count  $01f and + 1+ -2 and
;

: >name		\ cfa -- nfa
\ *G Move a pointer from an XT back to the NFA or name-pointer.
\ ** If the original pointer was not an XT or if the definition in
\ ** question has no name header in the dictionary the returned pointer
\ ** will be useless. Care should be taken when manipulating or scanning
\ ** the Forth dictionary in this way.
  begin
    1-  dup c@ $080 and
  until
;

internal
: (SEARCH-WORDLIST)     ( c-addr u ^^nfa -- 0 | xt 1 | xt -1 )
  begin
    @ dup
  while
    dup c@ $1F and			\ -- c-addr u ^nfa nfalen ;
    2 pick = if				\ Are the names the same *length*?
      dup 1+ 3 pick 3 pick s= if	\ Are they the same *name*?
        nip nip
        dup name> swap
        c@ $40 and ( 0= )		\ check for immediacy (0=immediate)
        if  -1  else  1  then
        exit
      then
    then
    ( n>link ) cell -
  repeat
  drop 2drop 0
;
external

: SEARCH-WORDLIST	\ c-addr u wid  --  0|xt 1|xt -1
\ *G Search the given wordlist for a definition. If the definition is
\ ** not found then 0 is returned, otherwise the XT of the definition
\ ** is returned along with a non-zero code. A -ve code indicates a
\ ** normal definition and a +ve code indicates an *\fo{IMMEDIATE} word.
\ a wid points to #threads, followed by that many threads.
  dup 0= if
    nip nip
  else
    over 3 pick c@ + over @ 1- and      \ -- c-addr u wid thread# ;
    1+ cells +                          \ -- c-addr u ^nfa ;
    (search-wordlist)
  then
;


\ \ ************************
\ \ *S VALUE and locals runtime
\ \ ************************
\
\ internal
\
\ ((
\ CODE VAL!       \ n -- ; store value, address in-line
\ \ *G Store n at the inline address following this word.
\   pop    r14			\ return address
\   mov    @r14+  r15		\ literal address
\   mov    tos  @r15		\ store
\   mov    @psp+  tos
\   br     r14
\ END-CODE
\
\ CODE VAL@       \ -- n ; read value, data address in-line
\ \ *G Read n from the inline address following this word.
\   pop    r14			\ return address
\   sub    # 2  psp
\   mov    tos  0 (psp)
\   mov    @r14+  r15		\ literal address
\   mov    @r15  tos
\   br     r14
\ END-CODE
\ ))
\ (( replaced by LIT
\ CODE VAL        \ -- a-addr ; read address of value offset in-line
\   std  2 ,-x                    \ save TOS
\   puly                          \ return address
\   ldd  0 ,y                     \ get address
\   jmp  2 ,y                     \ step over inline data
\ END-CODE
\ ))
\
\ external


\ *****************
\ *S Portability words
\ *****************
\ *P Using these words will make code easier to port between
\ ** 16, 32 and 64 bit targets.

cell constant cell	\ -- 2
\ *G Return the size in address units of one CELL.

: cells		\ n -- n*2
\ *G Return the number of bytes required to hold the given number
\ ** of 16 bit cells.
  2*
;

: aligned       \ addr -- addr'
\ *G Given an address pointer this word will return the next aligned
\ ** address subject to system wide alignment restrictions.
  1+ -2 and
;

: >body         \ xt -- pfa ; step from code field to parameter field
\ *G Move a pointer from a CFA or "XT" to the definition BODY. This
\ ** should only be used with children of *\fo{CREATE}. E.g. if *\fo{FOOBAR} is
\ ** defined with *\fo{CREATE foobar}, then the phrase *\fo{' foobar >body}
\ ** would yield the same result as executing foobar.
  4 + @
;

: compile,      \ xt --
\ *G Compile the word specified by xt into the current definition.
  $12B0 , ,				\ call # addr
;


\ *****************
\ *S Defining words
\ *****************

internal
: (dobuild,)	\ -- ; lay dummy cfa
  $FFFF compile,		\ lay down m/c call for later patching
;

: namedBuild,	\ val --
\ +G Start a target defining word.
  [ROM  header, (dobuild,) swap ,  ROM]
;

: (;CODE) 	\ -- ; R: a-addr --
\ +G Performed at compile time by *\fo{DOES>}. Patch the last
\ +* word defined (by *\fo{CREATE}) to have the run time actions
\ +* that follows immediately after *\fo{(;CODE)}.
  r> latest name>                       \ addr of dest
  $12B0 over !f  2+ !f			\ insert CALL # addr
;
external

: DOES>		\ C: colon-sys1 -- colon-sys2 ; Run: -- ; R:  nest-sys --
\ *G Begin definition of the runtime action of a child of a
\ ** defining word. You should not use *\fo{RECURSE} after
\ ** *\fo{DOES>}.
  COMPILE (;CODE)                       \ end of compile action
  $8324 ,				\ sub  # 2 psp
  $4584 ,  0 ,				\ mov  tos  0 (psp)
  $4135 ,				\ pop  tos
  $4525 ,				\ mov  @tos tos
;  IMMEDIATE

TARGET-ONLY
: :		\ C: "<spaces>name" -- colon-sys ; Exec: i*x -- j*x ; R: -- nest-sys
\ *G Begin a new definition called *\fo{name}.
  ( ?EXEC ) !CSP  header,  hide  [compile] ]
; immediate
HOST&TARGET

((
: BUFFER:	\ n "<spaces>name" --
 \ *G Create a named buffer in RAM of *\i{n} bytes.
  rhere constant  rallot
;
))

: CONSTANT 	\ x "<spaces>name" -- ; Exec: -- x
\ *G Create a new *\fo{CONSTANT} called *\fo{name} which has the
\ ** value "x". When *\fo{NAME} executes the value *\i{x) is
\ ** returned.
  namedBuild,
  ;CODE
    sub  # 2  psp		\ save previous TOS
    mov  tos 0 (psp)
    pop  r14			\ get address of data
    mov  @r14 tos		\ get data
    ret
end-code

: EQU 	\ x "<spaces>name" -- ; Exec: -- x
\ *G A synonym for *\fo{CONSTANT} abve to ease interactive
\ ** debugging of target drivers that are normally cross-compiled.
\ ** Create a new *\fo{CONSTANT} called *\fo{name} which has the
\ ** value "x". When *\fo{NAME} executes the value *\i{x) is
\ ** returned.
  constant
;

: VARIABLE 	\ "<spaces>name" -- ; Exec: -- a-addr
\ *G Create a new variable called *\fo{name}. When *\fo{Name} is executed the
\ ** address of the RAM is returned for use with *\fo{@} and *\fo{!}.
\ ** The RAM is not initialised.
  rhere namedBuild,  2 rallot
  ;CODE
    sub    # 2 psp
    mov    tos  0 (psp)
    pop    r15
    mov    @r15  tos
    ret
end-code

\ SFP009...
: USER 		\ u "<spaces>name" -- ; Exec: -- addr
\ *G Create a new *\fo{USER} variable called *\fo{name}. The 'u' parameter
\ ** specifies the index into the user-area table at which to place the
\ ** data.
\ ** *\fo{USER} variables are located in a separate area of memory for each
\ ** task or interrupt. Use in the form:
\ *C   $400 USER TaskData
  namedBuild,			\ ugly, but works
  ;CODE
    sub  # 2  psp		\ save previous TOS
    mov  tos 0 (psp)
    pop  r14			\ get address of offset
    mov  up tos			\ get USER area pointer
    add  @r14 tos		\ add offset
    ret
end-code
\ ...SFP009

interpreter
: u#		\ "<uservar>" -- offset ; u# <uservar>
\ +G Return the offset of a *\fo{USER} variable in the user area.
  ' >body
;
target

internal
: CRASH		\ --
\ +G The default action of a *\fo{DEFER}ed word. A *\fo{NOOP}
\ +* that satisfies the cross compiler.
\ +* INTERNAL.
\  #-256 throw
;
external

: DEFER 	\ Comp: "<spaces>name" -- ; Run:  i*x -- j*x
\ *G Creates a new *\fo{DEFER}ed word. No default action is
\ ** assigned. User-defined *\fo{DEFER}ed words *\b{must} be
\ ** initialised by the application before use.
\ *C   ' <action> IS <deferredword>
\ *P or (when compiled)
\ *C   ['] <action> IS <deferredword>
  rhere namedBuild,  2 rallot
  ;CODE
    pop  r14			\ followed by address of data in RAM
    mov  @r14 r15		\ get address in RAM
    br   @r15			\ go do it
end-code


\ ****************
\ *S Miscellaneous
\ ****************

defer pause	\ -- ; multitasker hook
\ *G Allows the sytem multitasker to get a look in. If the
\ ** multitasker has not been compiled, *\fo{PAUSE} is set
\ ** to *\fo{NOOP}.
  assign noop to-do pause

code reboot	\ --
\ *G Starts the watchdog and waits until the CPU reboots.
  mov	# 0 sr			\ no interrupts
  mov	# $5A00 & WDTCTL	\ restart watchdog
l: reboot1
  jmp	reboot1
end-code

: bor!		\ mask addr --
\ *G Set *\i{mask} bits in the byte at *\i{addr}.
  bor!  ;

: bbic!		\ mask addr --
\ *G Clear *\i{mask} bits in the byte at *\i{addr}.
  bbic!  ;

: btoggle!		\ mask addr --
\ *G Toggle the *\i{mask} bits in the byte at *\i{addr}.
  bxor!  ;

: btst		\ mask addr -- x
\ *G Return non-zero if the *\i{mask} bits in the byte at *\i{addr}
\ ** are non-zero.
  c@ and  ;

: or!		\ mask addr --
\ *G Set *\i{mask} bits in the word/cell at *\i{addr}.
  or!  ;

: bic!		\ mask addr --
\ *G Clear *\i{mask} bits in the word/cell at *\i{addr}.
  bic!  ;

: toggle!		\ mask addr --
\ *G Toggle the *\i{mask} bits in the word/cell at *\i{addr}.
  xor!  ;

: tst		\ mask addr -- x
\ *G Return non-zero if the *\i{mask} bits in the word/cell at *\i{addr}
\ ** are non-zero.
  @ and  ;


\ ======
\ *> ###
\ ======

decimal
