\ LedTick2553.fth - timer using the watchdog unit and LED flasher

((
Copyright (c) 2007, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ ==========
\ *! ledticklp2553
\ *T Ticker using watchdog timer
\ ==========
\ *P The ticker interrupt is provided in *\i{LedTickLP2553.fth}.

[undefined] ticks [if]
variable <ticks>

: ticks		( -- n )  <ticks> @  ;
\ *G Return current clock value in milliseconds. This can
\ ** be treated as a 16 bit unsigned value that will wrap
\ ** when it overflows.
compiler
: ticks		( -- n )  <ticks> @  ;
target
[endif]

system-speed 32768 /
  equ TickHz	\ -- hz
\ *G Ticker speed in Herz
1000 TickHz / equ Tick-Ms	\ -- ms
\ *G Milliseconds per tick.

variable LedActive	\ -- addr
\ *G Set true for LEDs to flash on the timer. Set this to false
\ ** (zero) when you want to use the LEDs yourself.
  -1 LedActive !
internal
variable LedTimer	\ -- addr
\ +G Holds LED timer value.
external

Proc WDT-isr
\ *G Assembler coded interrupt service routine.
\ *[
  add .w    # Tick-ms  & <ticks>
  add .w    # Tick-ms  & LedTimer
  cmp       # #1000 & LedTimer		\ 1 second timeout
  eq, if,
    mov .w    # 0 & LedTimer		\ reset timer
    cmp       # 0  & LedActive		\ if enabled
    ne, if,
      xor .b    # $41 & P1OUT		\ toggle LEDs
    endif,
  endif,
  reti
end-code
WDT-isr WDT_vec !
\ *]

: Start-Clock	\ --
\ *G Start the ticker interrupt.
  $5A10 WDTCTL !			\ interval timer, SMCLK, /32768
  $01 IFG1 bbic!			\ clear WDTIFG
  $01 IE1 bor!				\ enable WDT interrupt
  LedActive @ if
    $40 P1OUT bor!
    $01 P1OUT bbic!
  endif
  ei
;

: Stop-Clock	\ --
\ *G Stop the ticker interrupt
  $01 IE1 bbic!				\ disable WDT interrupt
;

: green-on	\ -- ; P1.6
\ *G Turn green LED on.
  $40 P1OUT bor!  ;
: green-off	\ -- ; P1.6
\ *G Turn green LED off.
  $40 P1OUT bbic!  ;
: red-on	\ -- ; P1.0
\ *G Turn red LED on.
  $01 P1OUT bor!  ;
: red-off	\ -- ; P1.0
\ *G Turn red LED off.
  $01 P1OUT bbic!  ;


\ ======
\ *> ###
\ ======

decimal
