\ Interrupt driven serial I/O words for MSP430 systems

((
Copyright (c) 2002, 2003, 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Start with USART0

To do
=====
Baud rate calculator
Proper initialisation

Change history
==============
20040105 RGB001 MODULATOR_TAB and SET-BAUD added.
		INIT-SER modified to support full range of devices.
		and to use SYSBAUD and SYSMOD equates.
))

decimal

\ **********
\ *! ser430i
\ **********
\ *T MSP430 serial driver
\ *P The file *\i{MSP430\Drivers\Ser430i.fth} contains the code
\ ** for a serial driver that uses an interrupt driven receiver
\ ** an a polled transmitter.


\ **********************
\ *S Baud rate calculations
\ **********************

((
  Divider = subsystem clock / console-speed
))

create MODULATOR_TAB	\ -- addr ; RGB001
\ *G Table of values to use for the MSP430 MCTL modulator
\ ** register calculated from the fraction left behind after
\ ** dividing BRCLK by the desired baud-rate. Larger errors
\ ** require more BRCLK cycles to be inserted. Note that this
\ ** table assumes 8 data bits, 1 stop-bit and no parity,
\ ** i.e. 10 bits per character
\ < 0.1%  0.1%    0.2%    0.3%    0.4%
  $00 c,  $10 c,  $90 c,  $98 c,  $9A c,
\ 0.5%    0.6%    0.7%    0.8%    0.9%
  $9A c,  $BA c,  $BE c,  $FE c,  $FF c,


host-math
SMCLK-speed console-speed /mod equ sysbaud
\ *G Calculate integer portion of baud-rate divider, assuming that
\ ** BRCLK is sourced from SMCLK.
10 console-speed */ 9 min MODULATOR_TAB + c@ equ sysmod
\ *G Calculate fraction portional of baud-rate divider from the
\ ** remainder, which is in the range 0..CONSOLE-SPEED-1.

USSEL1 USSEL0 or equ UART_CLK_MASK

: set-baud	\ BPS -- ; set UART baud-rate
\ *G Sets the baud-rate of UART0 appropriate for clock source
\ ** used for BRCLK.
  dup

\ Obtain current UART clock source and use CASE statement to return speed of
\ that clock source
  U0TCTL c@ UART_CLK_MASK and
  case
    $00 of  [ U0CLK-speed $10000 /mod swap ] literal literal  endof
    $10 of  [ ACLK-speed  $10000 /mod swap ] literal literal  endof
            [ SMCLK-speed $10000 /mod swap ] literal literal
   rot
  endcase

  rot um/mod                    \ divide clock speed by baud-rate
  [asm dint asm]                \ disable interrupts before updating registers
  $100 /mod U0BR1 c!            \ set baud-rate divider MSB
  U0BR0 c!                      \ set baud-rate divider LSB

  #10 rot */ 9 min              \ calculate baud-rate calculation error
  MODULATOR_TAB + c@            \ fetch correction factor
  U0MCTL c!                     \ .. and update UART modulator register
  [asm eint asm]                \ enable interrupts
;

target-math


\ ***************************
\ *S Interrupt enable/disable
\ ***************************
\ *P These words are defined here only if they are not yet present.

[undefined] di [if]
code di		\ --
\ *G Global interrupt disable.
  dint  ret
end-code
[then]

[undefined] ei [if]
code ei		\ --
\ *G Global interrupt enable.
  eint  ret
end-code
[then]


\ ******************
\ *S UART primitives
\ ******************

internal

variable rx0-char	\ -- addr
\ *G Low byte holds KEY character, high bye holds KEY? flag.
rx0-char 1+ equ rx0-avail	\ -- addr
\ *G Address of KEY? flag.

proc Rx0-isr	\ --
\ *G UART0 receive interrupt handler.
  mov .b  & u0rxbuf  & rx0-char		\ stash character
  mov .b  # $FF  & rx0-avail		\ flag character available
  bic     # _cpuoff  0 (sp)		\ return with cpu active
  reti
end-code
rx0-isr USART0rx_vec !			\ set vector

13x_or_14x? [if]
code init-ser	\ --
\ *G Initialise the serial port to the default baud rate
\ ** *\fo{CONSOLE-SPEED}. The version compiled depends on
\ ** whether the equate *\fo{13x_or_14x?} is non-zero.
  dint
\ set up P3 for USART0
  or .b   # $30 & p3sel                 \ P3.4,5 = USART0 TXD/RXD
  bic .b  # $20 & p3dir			\ P3.5 input direction
  or .b   # $10 & p3dir                 \ P3.4 output dirction
\ Set up USART and driver
  mov     # 0 & rx0-char		\ no characters received yet
  bic .b  # $C0 & ie1			\ disable USART0 TX and RX interrupts
  or .b   # $C0 & me1                   \ enable USART0 TX and RX modules
  mov .b  # $11 & u0ctl			\ 8N1, SWRST
  mov .b  # $30 & u0tctl                \ baud rate source is SMCLK
  mov .b  # $08 & u0rctl		\ receive all characters
  mov .b  # sysbaud .lo & u0br0		\ baud-rate LSB ; RGB001
  mov .b  # sysbaud .hi & u0br1		\ baud-rate MSB ; RGB001
  mov .b  # sysmod .lo & u0mctl		\ baud-rate modulator
  bic .b  # $01 & u0ctl			\ clear SWRST, must be before int. enable
  bis .b  # $40 & ie1			\ enable UART0 RX interrupt
  eint                  		\ global enable interrupts
  ret
end-code
[else]
code init-ser	\ --
  or .b   # $03 & me2                   \ enable USART0 tx & rx ; RGB001
  or .b   # $30 & u0tctl                \ baud rate source is smclk
  mov .b  # sysbaud & u0br0             \ baud-rate LSB ; RGB001
  mov .b  # sysbaud 8 rshift & u0br1    \ baud-rate MSB ; RGB001
  mov .b  # sysmod & u0mctl             \ baud-rate modulator
  mov .b  # $30 & u0ctl			\ 8N2, clear reset
  or .b   # $30 & p3sel                 \ P3.4,5 = USART0 TXD/RXD
  or .b   # $10 & p3dir                 \ P3.4 output dirction
  mov     # 0 & rx0-char		\ no characters received yet
  bis .b  # $01 & ie2                   \ enable UART 0 receive interrupt ; RGB001
  bic .b  # $80 & BCSCTL1		\ XT2 = HF XTAL
  bis .b  # $88 & BCSCTL2
  eint                  		\ global enable interrupts
  ret
end-code
[then]

: key?0		\ -- flag
\ *G Return true if USART0 received a character.
  rx0-avail c@ 0<>
;

: key0		\ -- char
\ *G Wait for character from USART0 and return it.
  begin
    di  rx0-avail c@ 0=
   while
    ei pause				\ schedule
  repeat
  rx0-char c@  0 rx0-avail c!
  ei
;

: emit0		\ char --
\ *G Send a character through USART0.
  begin  u0tctl c@ 1 and  until		\ wait for empty buffer
  u0txbuf c!				\ write buffer
;

: type0		\ c-addr len --
\ *G Send a string through USART0.
  bounds
  ?do  i c@ emit0  loop
;

: cr0		\ --
\ *G Perform CR on USART0.
  $0D emit0  $0A emit0
;

external


\ ***********************
\ *S Generic i/o assignments
\ ***********************

cdata
create Console0	\ -- addr ; OUT managed by upper driver
\ *G The device vector for USART0.
  ' key0 ,		\ -- char ; receive char
  ' key?0 ,		\ -- flag ; check receive char
  ' emit0 ,		\ -- char ; display char
  ' type0 ,		\ caddr len -- ; display string
  ' cr0 ,		\ -- ; display new line

console-port 0 = [if]
  Console0 constant Console
\ *G Defines USART0 as the default system console.
[then]


\ ******
\ *> ###
\ ******

decimal

