\ Interrupt driven serial I/O words for MSP430 systems

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Start with USART0

To do
=====
Baud rate calculator
Proper initialisation

Change history
==============
))


\ **********************
\ Baud rate calculations
\ **********************

((
  Divider = subsystem clock / console-speed
))


\ *********
\ UART code
\ *********

variable rx0-char	\ -- addr ; low byte holds KEY character
rx0-char 1+
  equ rx0-avail		\           high bye holds KEY? flag

proc Rx0-isr	\ --
  mov .b  & u0rxbuf  & rx0-char		\ stash character
  mov .b  # -1  & rx0-avail		\ flag character available
  bic     # _cpuoff 0 (sp)		\ return with cpu active
  reti
end-code
rx0-isr USART0rx_vec !			\ set vector

code init-ser	\ --
  or .b   # $c0 & me1                   \ enable USART0 tx and rx
  or .b   # $30 & u0tctl                \ baud rate source is smclk
  mov .b  # $a1 & u0br0                 \ 9600 baud
  mov .b  # $01 & u0br1
  mov .b  # $03 & u0mctl
  mov .b  # $30 & u0ctl			\ 8N2, clear reset
  or .b   # $30 & p3sel                 \ P3.4,5 = USART0 TXD/RXD
  or .b   # $10 & p3dir                 \ P3.4 output dirction
  mov     # 0 & rx0-char		\ no characters received yet
  bis .b  # $40 & ie1                   \ enable UART 0 receive interrupt
  bic .b  # $80 & BCSCTL1       	\ XT2 = HF XTAL
  bis .b  # $88 & BCSCTL2
  eint                  		\ global enable interrupts
  ret
end-code

: key?0		\ -- flag ; true if USART0 received a character
  rx0-avail c@ 0<>
;

: key0		\ -- char ; wait for character from USART0
  begin
    di  rx0-avail c@ 0=
   while
    ei pause				\ schedule
\    [asm  bis # _cpuoff _gie + sr  asm]	\ cpu to sleep, GIE set
  repeat
  rx0-char c@  0 rx0-avail c!
  ei
;

: emit0		\ char -- ; send character through USART0
  begin  u0tctl c@ 1 and  until		\ wait for empty buffer
  u0txbuf c!				\ write buffer
;

: type0		\ c-addr len --
  bounds
  ?do  i c@ emit0  loop
;

: cr0		\ --
  $0D emit0  $0A emit0
;


\ ***********************
\ Generic i/o assignments
\ ***********************

internal

cdata
create Console	\ -- addr ; OUT managed by upper driver
  ' key0 ,		\ -- char ; receive char
  ' key?0 ,		\ -- flag ; check receive char
  ' emit0 ,		\ -- char ; display char
  ' type0 ,		\ caddr len -- ; display string
  ' cr0 ,		\ -- ; display new line

decimal

