\ TickWDT.fth - timer using the watchdog unit.

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ ==========
\ *! tickwdt
\ *T Ticker using watchdog timer
\ ==========
\ *P Two versions of the ticker are provided in *\i{TickWDT.fth}.
\ ** By default the low-level interrupt handler is used as it is
\ ** faster. The high level version is provided as an example of
\ ** using the interrupt facilities in *\i{Intrpt430.fth}.

[undefined] ticks [if]
variable <ticks>

: ticks		( -- n )  <ticks> @  ;
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 16 bit unsigned value that will wrap
\ ** when it overflows.
compiler
: ticks		( -- n )  <ticks> @  ;
target
[endif]


\ ********************
\ *S low level version
\ ********************

timebase? 0= [if]
Proc WDT-isr
\ *G Assembler coded interrupt service routine.
\ *[
  add .w    # 16  & <ticks>
  reti
end-code
WDT-isr WDT_vec !
\ *]
[then]


\ *********************
\ *S High level version
\ *********************
\ *P This version requires *\i{Intrpt430.fth} to be compiled before
\ ** this file. The high level version is *\b{much} slower, and should
\ ** only be used if interrupt latency is not an issue.

timebase? [if]
: ticker-isr	\ --
\ *G Ticker interrupt handler.
  #16 addr ticks +!  do-timers
;

Proc TIMER-WDT
\ *G Assembler wrapper for the Forth handler.
   push   r15
   mov .w # ' ticker-isr  r15
   jmp    next-int
End-Code

TIMER-WDT wdt_vec !
\ *G Set the relevant interrupt vector.
[then]


\ **********************************
\ *S Starting and stopping the clock
\ **********************************

: Start-Clock	\ --
\ *G Start the ticker interrupt.
  $5A16 WDTCTL !			\ WDT 16ms, ACLK, interval timer
  $01 IE1 bor!				\ enable WDT interrupt
  ei
;

: Stop-Clock	\ --
\ *G Stop the ticker interrupt
  $01 IE1 bbic!				\ disable WDT interrupt
;


\ ***********************
\ *S Time base extensions
\ ***********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  init-timers  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

decimal
