\ Virtual UART for MSP430 systems

((
Copyright (c) 2003, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
))


\ ****************
\ VUART allocation
\ ****************

\ *** DO NOT CHANGE ***
\ Offsets to registers used by FET comms link.
\ RO = host read-only
\ WO = host write-only
\ RW = host read/write
0 equ data(h>t)			\ WO, host to target data
1 equ status(h>t)		\ RW, host to target status
2 equ data(t>h)			\ RO, target to host data
3 equ status(t>h)		\ RW, target to host status
4 equ /VUART			\ size of VUART data block

idata
here is-VUART  /VUART allot		\ reserve space
cdata


\ *********
\ UART code
\ *********

: initVuart	\ --
\ *G Initialise the VUART.
  VUART /VUART 0 fill
;

: key?v		\ -- flag
\ *G Return true if the VUART has received a character.
  VUART status(h>t) + c@ $FF =
;

: keyv		\ -- char
\ *G Wait for a character from the VUART.
  begin  key?v until
  VUART
  dup data(h>t) + c@ swap		\ -- char VUART
  0 swap status(h>t) + c!		\ acknowledge
;

: emitv		\ char --
\ *G Send a character to the VUART.
  VUART					\ -- char VUART
  begin					\ wait for previous char
    dup status(t>h) + c@ 0=
  until
  tuck data(t>h) + c!			\ stash character
  -1 swap status(t>h) + c!		\ flag data available
;

: typev		\ c-addr len --
\ *G Type a string to the VUART
  bounds
  ?do  i c@ emitv  loop
;

: crv		\ --
\ *G Send CR/LF through the VUART.
  $0D emitv  $0A emitv
;


\ ***********************
\ Generic i/o assignments
\ ***********************

internal

cdata
create ConsoleV	\ -- addr ; OUT managed by upper driver
  ' keyv ,		\ -- char ; receive char
  ' key?v ,		\ -- flag ; check receive char
  ' emitv ,		\ -- char ; display char
  ' typev ,		\ caddr len -- ; display string
  ' crv ,		\ -- ; display new line

console-port #10 = [if]
  ConsoleV constant Console
[then]

decimal

