\ Port I/O
\ Created by Duncan February 2014

\ ==========
\ *> drivers
\ ==========

\ *******************
\ *S Basic port usage
\ *******************
\ *P See the file *\i{Drivers\gpio2553.fth}.
\ *P The user words are as follows:
\ *(
\ *B *\fo{HI} and *\fo{LO} configure the port bit for output.
\ *B *\fo{BIT?} does not configure the port bit for input.
\ *B *\fo{SETIN} configures the port bit for input.
\ *)

P1IN CONSTANT PORT1	\ -- addr
\ *G Base of Port 1.
P2IN CONSTANT PORT2	\ -- addr
\ *G Base of Port 2

INTERNAL
CODE PORT?	( addr --- f )
\ +G Return true if addr is PORT1 or PORT2.
	MOV 	TOS R13	\ Copy the TOS
	CLR	TOS		\ Clear flag
	CMP	# PORT1 R13
	EQ, IF,
		MOV # -1 TOS
	THEN,
	CMP	# PORT2 R13
	EQ, IF,
		MOV # -1 TOS
	THEN,
	RET
END-CODE

[undefined] LSHIFT [if]
code lshift	\ x count -- x'
\ +G Left shift x by count bits.
  mov    tos  r14			\ R14 := count
  mov    @psp+  tos			\ TOS := x
  cmp    # 0 r14			\ count != 0 ?
  ne, if,
    begin,
      add    tos  tos
      sub    # 1 r14
    eq, until,
  endif,
  ret
end-code
[then]

: >MASK	( n --- mask )
\ +G Convert a bit number to a bit mask.
	1 SWAP LSHIFT ;

: SETOUTPUTETC	( bit port --- mask address )
\ +G Set pin to output mode. Deselects other functions.
	SWAP >MASK SWAP 2DUP 2+ BOR!
	2DUP 6 + BBIC!			\ Reset sel bit
	 1+ ;
EXTERNAL

: HI		\ bit port --
\ *G Set bit high.
	DUP PORT?
	IF SETOUTPUTETC BOR!
	ELSE 2DROP
	THEN ;

: LO		\ bit port --
\ *G Set bit low.
	DUP PORT?
	IF SETOUTPUTETC BBIC!
	ELSE 2DROP
	THEN ;

: SETIN		\ bit port --
\ *G Set pin to input. Deselect other functions.
	SWAP >MASK SWAP 2DUP 2+ BBIC! 6 + BBIC! ;

: BIT?		\ bit port --- f
\ *G Return the state of the bit in the port.
	C@ SWAP >MASK AND 0<> ;


\ *********************************
\ *S Port counting using interrupts
\ *********************************
\ *P See the file *\i{Drivers\gpio2553.fth}.
\ *P The notation is:
\ *C   bit port startcount
\ *P The code assumes that interrupts are enabled elsewhere.

PORT1 3 + EQU P1IF
PORT2 3 + EQU P2IF

VARIABLE P1COUNT	\ -- addr
\ *G Holds the count for Port 1.

PROC PORT1-ISR
	ADD .W # 1 & P1COUNT
	MOV .B # 0 P1IF		\ Clear interrupt flag
	RETI
END-CODE
PORT1-ISR P1_VEC !

VARIABLE P2COUNT	\ -- addr
\ *G Holds the count for Port 2.

PROC PORT2-ISR
	ADD .W # 1 & P2COUNT
	MOV .B # 0 P2IF		\ Clear interrupt flag
	RETI
END-CODE
PORT2-ISR P2_VEC !

INTERNAL
: (STARTCOUNT)	( bit port -- )
	0 OVER 3 + C!			\ Clear any pending interrupt flag.
	0 OVER 4 + C!			\ Make sure rising edges selected.
	SWAP >MASK SWAP 5 + C! ;	\ Only one bit enabled at a time.
EXTERNAL

: STARTCOUNT	( bit port -- )
\ *G Start counting transitions for the given bit and port.
	DUP PORT? IF (STARTCOUNT) ELSE 2DROP THEN ;

: STOPCOUNT	( port -- )
\ *G Stop counting transitions on the given port.
	0 SWAP 5 + C! ;


\ ======
\ *> ###
\ ======
