\ simple PWM driver - to be cross-compiled
\ Duncan Louttit 1/2/14

\ ==========
\ *> drivers
\ ==========
\ *S PWM
\ ******
\ *P See the file *\i{Drivers\pwm2553.fth}.
\ ** The code sets up for three possible channels of 10 bit PWM
\ ** for 20 pin device. The PWM frequency is about 7.5KHz.

\ *P Use *\fo{P1.6PWM}, *\fo{P2.1PWM} or *\fo{P2.4PWM} to
\ ** initialise the relevant port and timer.
\ ** P1.6 uses Timer0, the other two pins use Timer1.
\ ** It is best to use P2.1 and P2.4 first to keep
\ ** the other timer free.
\ ** Once initialised, use *\fo{P1.6DUTY}, *\fo{P2.1DUTY} or
\ ** *\fo{P2.4DUTY} to set the duty cycle.

$0022 EQU P1DIR
$0026 EQU P1SEL

$002A EQU P2DIR
$002E EQU P2SEL

\ Timer0 PWM for P1.6

$0174 EQU T0TACCR1
$0172 EQU T0TACCR0
$0160 EQU T0TACTL
$0164 EQU T0TACCTL1

: P1.6DUTY	\ value --
\ *G Set the duty cycle on P1.6, $0..3FF.
	T0TACCR1 ! ;

: P1.6PWM	\ duty --
\ *G Start PWM on pin 1.6, where *\i{duty} is $0..3FF
	$3FF T0TACCR0 !
	$210 T0TACTL !
	$E0  T0TACCTL1 !
	$40 P1SEL BOR!
	$40 P1DIR BOR!
	P1.6DUTY ;

\ Timer1 pwm for P2.1 AND P2.4
$0196 EQU T1TACCR2
$0194 EQU T1TACCR1
$0192 EQU T1TACCR0
$180 EQU T1TACTL
$186 EQU T1TACCTL2
$184 EQU T1TACCTL1

: P2.4DUTY	( value --- )
	T1TACCR2 ! ;

: P2.1DUTY	( value --- )
	T1TACCR1 ! ;

INTERNAL
: STARTPWM1	( --- )
	$3FF T1TACCR0 !
	$210 T1TACTL ! ;
EXTERNAL

: P2.1PWM	\ duty --
\ *G Start PWM on pin 2.1, where *\i{duty} is $0..3FF
	STARTPWM1
	$E0 T1TACCTL1 !
	2 P2SEL BOR!
	2 P2DIR BOR!
	P2.1DUTY ;

: P2.4PWM	\ duty --
\ *G Start PWM on pin 2.4, where *\i{duty} is $0..3FF
	STARTPWM1
	$E0 T1TACCTL2 !
	$10 P2DIR BOR!
	$10 P2SEL BOR!
	P2.4DUTY ;

\ ======
\ *> ###
\ ======

