\ serUSCIp.fth - polled serial driver

((
Copyright (c) 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Change history
==============
20141209 SFP003 Removed channel mechanism.
20120824 MPE002 Conversion for USCI and G2553
20040105 RGB001 MODULATOR_TAB and SET-BAUD added.
		INIT-SER modified to support full range of devices.
		and to use SYSBAUD and SYSMOD equates.
))

decimal

\ ***********
\ *! seruscip
\ ***********
\ *T USCI serial driver
\ *P The file *\i{MSP430Lite\Drivers\serUSCIp.fth} contains the code
\ ** for a polled serial driver.


\ ************************
\ *S Baud rate calculation
\ ************************

((
  Divider = subsystem clock / console-speed
))
host-math  interpreter
: genUSCIlf	\ baud clock -- brx brsx
\ *G Generate the baud rate values for low frequency baud rate
\ ** settings.
\ ** The word can only be executed while interpreting in the cross
\ ** compiler.
  16 * swap / 1+			\ quotient is rounded up.
  dup 4 rshift swap			\ integer brx
  1 rshift $07 and			\ fraction brs
;

: genUSCIos	\ baud clock -- brx brfx
\ *G Generate the baud rate values for oversampling baud rate
\ ** settings.
\ ** The word can only be executed while interpreting in the cross
\ ** compiler.
  32 * swap / 1+			\ quotient is rounded up.
  dup 5 rshift swap			\ integer brx
  1 rshift $0F and			\ fraction brfx
;
target  target-math


\ ********
\ *S UART0
\ ********

: key?		\ -- flag
\ *G Return true if UART0 received a character.
  IFG2 c@ bit0 and 0<>
;

: key		\ -- char
\ *G Wait for character from UART0 and return it.
  begin
    pause  IFG2 c@ bit0 and
  until
  UCA0RXBUF c@
;

: emit		\ char --
\ *G Send a character through UART0.
  begin  IFG2 c@ bit1 and  until	\ wait for empty buffer
  UCA0TXBUF c!				\ write buffer
  1 out +!
;

: type		\ c-addr len --
\ *G Send a string through UART0.
  bounds
  ?do  i c@ emit  loop
;

: cr		\ --
\ *G Perform CR on UART0.
  $0D emit  $0A emit  out off
;

console0-speed system-speed genUSCIlf  equ BRS0  equ BRX0
\ *G Generate the baud rate values for UART0.

\ Initializing or Re-Configuring the USCI Module
\ The recommended USCI initialization/re-configuration process is:
\ 1. Set UCSWRST (BIS.B #UCSWRST,&UCAxCTL1)
\ 2. Initialize all USCI registers with UCSWRST = 1 (including UCAxCTL1)
\ 3. Configure ports.
\ 4. Clear UCSWRST via software (BIC.B #UCSWRST,&UCAxCTL1)
\ 5. Enable interrupts (optional) via UCAxRXIE and/or UCAxTXIE)

: init-ser	\ --
\ *G Configure UART0. This word can be hardware dependent. The
\ ** version here assumes USCI A0 on P1.1 and P1.2.
  6 P1SEL bor!				\ Function select bits for P1.1 and P1.2
  6 P1SEL2 bor!
  $081 UCA0CTL1 c!			\ SMCLK and reset
  $000 UCA0CTL0 c!			\ 8N1 UART, LSB first
  BRX0 $FF and UCA0BR0 c!		\ low part of baud rate integer
  BRX0 8 rshift UCA0BR1 c!		\ high part of baud rate integer
  BRS0 1 lshift UCA0MCTL c!		\ fraction, UCOS16=0
  $080 UCA0CTL1 c!			\ SMCLK and no reset
;

external


\ ******
\ *> ###
\ ******

decimal

