\ DebugChan.fth - using AIDE's debug display channels

\ Copyright (c) 2014
\ MicroProcessor Engineering
\ 133 Hill Lane
\ Southampton SO15 5AF
\ England
\
\ tel:   +44 (0)23 8063 1441
\ fax:   +44 (0)23 8033 9691
\ email: mpe@mpeforth.com
\        tech-support@mpeforth.com
\ web:   http://www.mpeforth.com
\ Skype: mpe_sfp
\
\ From North America, our telephone and fax numbers are:
\        011 44 23 8063 1441
\        011 44 23 8033 9691
\        901 313 4312 (North American access number to UK office)
\
\
\ To do
\ =====
\
\ Change history
\ ==============


decimal

\ ===========
\ *! debugchan
\ *T AIDE Auxiliary Debug Displays
\ ===========
\ *P From AIDE 6.6 onwards, AIDE's PowerTerm can provide
\ ** additional debug displays. These are separate from the
\ ** main PowerTerm console, and behave like traditional
\ ** cursor-addressable units with a fixed-width font.

\ *P The displays are controlled by sending control sequences
\ ** starting with <ESC><!>. The whole of the command sequence
\ ** must be performed with inter-character gaps less than
\ ** 100 ms.
\ *E <ESC>!'0'       Use the main PowerTerm console - char 0
\ ** <ESC>!'1'       Use the the first debug display - char 1
\ ** <ESC>!'2'       Use the the second debug display - char 2
\ ** <ESC>!...
\ ** <ESC>!'8'       Use the the eighth debug display - char 8
\ **                 If a display is not open, it is created.
\ **                 If it has not been defined, the default is
\ **                 80x25 in a 10 point font.
\ ** <ESC>!'9'n      Close display n (1..8).
\ ** <ESC>!'10'nwhf  Define the display size and open it
\ **                 n is the display in the range 1..8
\ **                 w (width) and h (height) are 1..255.
\ **                 f (font height) is 6..24
\ **                 The display is not selected for output.
\ ** <ESC>!'11'n#<txt>  Set the caption of an open display
\ **                 n is the display, 1..8
\ **                 # is the number of characters in the string
\ **                 txt is the string (no terminator)
\ ** <ESC>!'12'nfffbbb Set the foreground and background colours
\ **                 n is the display in the range 1..8
\ **                 fff is a 24 bit foreground colour
\ **                 bbb is a 24 bit background colour
\ ** <ESC>!'13'n     Clear the display to the background colour
\ **                 n is the display in the range 1..8
\ ** <ESC>!'14'nxy   Goto the given position
\ **                 n is the display in the range 1..8
\ **                 x is the X position from the left
\ **                 y is the Y position from the top
\ ** <ESC><ESC>      Pass a single Escape character to the host.

\ *P An example of using these displays can be found in
\ ** *\i{Examples/hanoi.fth}.
\ ** *\image{"Towers of Hanoi" "HanoiBig" "png" 80 80}

: esc!		\ --
\ *G Send escape prefix.
  27 emit  [char] ! emit  ;

semaphore SerChannel  1 SerChannel !	\ -- addr
\ *G Exclusive access semaphore.

: [d		\ disp -- ; disp=0..8
\ *G Select/Launch the given display. The auxiliary consoles are
\ ** numbered 1..8. This is the multi-tasking version which
\ ** includes a semaphore for the serial channel.
  SerChannel request  esc!  emit
;
: d]		\ --
\ *G End a display sequence and return to the main console.
\ ** This is the multi-tasking version which
\ ** includes a semaphore for the serial channel.
  esc!  0 emit  SerChannel signal
;

: escCmd	\ disp fn# --
\ Output the function number and then the display number
  esc!  emit  emit
;

: dclose	\ disp -- ; disp=0..8
\ *G Close the given display. If this is the current display,
\ ** the main display is used.
\  <ESC>!'9'n      Close display n (1..8).
  9 escCmd
;

: dopen		\ w h f disp -- ; disp=1..8
\ *G Open an auxiliary display with width=w, height=h and font
\ ** size f points. The display is not selected.
\  <ESC>!'10'nwhf
  10 escCmd  rot emit  swap emit  emit
;

: dcaption	\ caddr len disp -- ; disp=1..8
\ *G Set the string as the caption of the display.
\ <ESC>!'11'n#<txt>  Set the caption of an open display
  11 escCmd  dup emit  type
;

: d24		\ x --
\ Send as a 24 bit colour.
  dup 16 rshift emit  dup 8 rshift emit  emit
;

: dcolours	\ forecolor backcolor disp -- ; disp=0..8
\ *G Set the foreground and background colours of the given
\ ** display. Some colours will cause problems on the console
\ ** (disp=0). Colours are of the form *\f{00bbggrr}.
\  <ESC>!'12'nfffbbb Set the foreground and background colours
  12 escCmd  swap d24  d24
;

\ *P A few standard colours:
\ *[
$000000 constant RGB_BLACK	\ -- colorref
$DFDFDF constant RGB_LTGRAY	\ -- colorref
$FFFFFF constant RGB_WHITE	\ -- colorref
$0000FF constant RGB_RED	\ -- colorref
$00FF00 constant RGB_GREEN	\ -- colorref
$FF0000 constant RGB_BLUE	\ -- colorref
\ *]

: dclear	\ disp -- ; disp=0..8
\ *G Clear the display.
\  <ESC>!'13'n     Clear the display to the background colour
  13 escCmd
;

: datxy		\ x y disp -- disp=1..8
\ *G Goto the given position. Does not work on the console (disp=0).
\  <ESC>!'14'nxy   Goto the given position
  14 escCmd  swap emit emit
;

((
: topen3	\ --
  80 25 8 3 dopen
  $FF0000 $00FF00 3 dcolours
  3 dclear  3 [d
;

: topen4	\ --
  80 25 10 4 dopen
  4 dclear  4 [d
;
))


\ ======
\ *> ###
\ ======

decimal
