\ hanoi.fth - Towers of hanoi

((
A Famous Forth implementation of a classic problem. The original
version was written by Peter Midnight. This version was
converted to PowerForth/386 by MPE from FORTH DIMENSIONS, Vol II,
No. 2, page 32. It was then converted to VFX Forth, and then
to target code using AIDE's cursor-addressable displays.

This is also a good demonstration of the use of RECURSE. If you
observe a crash after running TOWERS, you probably need to
increase the stack sizes.

To adjust the display to the speed of your CPU use:
  n SET-SPEED
where 5000 is a good starting point.

To run the demo, use
  n TOWERS
where n is the number of rings, e.g. 5.

To stop the display, press any key.

This code can be cross-compiled, or it can be downloaded
directly onto the target. To download directly, enable the
file server on AIDE, make sure that the target has INCLUDE
and that Common/DebugChan.fth has been compiled, then
  include hanoi.fth
))

decimal

12 constant nmax	\ -- n ; max number of rings.
3 nmax min value n	\ -- n ; number of rings.
5000 value speed	\ -- delay
nmax 2 + buffer: ring	\ array [1...n] of bytes

0 constant hell.freezes.over	\ see later
char | constant v-bar		\ vertical bar character (pole)
char = constant base-char	\ base character
char o constant ring-char	\ ring character

4 value HanoiTerm	\ -- n
\ *G Define the AIDE auxiliary display used to display output.
\ ** Do not use 0.

: OpenHanoi	\ --
\ *G Set up and switch to the auxiliary display.
  3 nmax min to n
  5000 to speed
  4 to HanoiTerm
  ring nmax 2 + erase
  80 25 10 HanoiTerm dopen
  RGB_GREEN RGB_BLACK HanoiTerm dcolours
  HanoiTerm dclear
  s" Towers of Hanoi" HanoiTerm dcaption
  HanoiTerm [d
;

: CloseHanoi	\ --
\ *G Close output, but leave display active.
  d]
;

: cls		\ --
\ *G Clear the screen/display.
  HanoiTerm dclear  ;

: set-speed	\ n --
\ *G Set the target delay value, higher=slower.
  to speed  ;

: delay		\ n --
\ *G Waste time by calling PAUSE a lot.
  0 ?do				\ outer loop
    speed 0
    ?do  pause  loop		\ and waste time
  loop
;

: h-emit	\ char --
\ *G Print and wait a little bit.
  emit  1 delay  ;

: 4dup		\ n1 n2 n3 n4 -- n1 n2 n3 n4 n1 n2 n3 n4
\ *G Duplicate 4 items.
  3 pick  3 pick  3 pick  3 pick
;

: at-xy		\ x y --
\ *G Position the cursor.
  HanoiTerm datxy
;

: pos		\ location tower -- coordinate
  n 2* 1+ *  n +  ;

: halfdisplay	\ char size --
  0 ?do  dup h-emit  loop
  drop
;

: <display>	\ line char size --
  2dup halfdisplay rot 3 <
  if  bl  else  v-bar  then
  h-emit  halfdisplay
;

: display	\ size pos line char --
  swap >r
  -rot over - r@		\ -- color size pos line
  at-xy
  r>				\ -- color size line
  -rot <display>
;

: presence	\ tower ring -- 0/1
  ring + c@ = 1 and
;

: line		\ tower line -- display-line-of-top
  4 swap  n 0 do
    dup i presence 0= 1 and	\ convert now add boolean (yuk)
    rot + swap
  loop
  drop
;

: raise		\ size tower --
  dup pos swap line
  2 swap do
    2dup i bl display
    2dup i 1- ring-char display
  -1 +loop
  2drop
;

: lower		\ size tower --
  dup pos  swap line
  1+ 2 do
    2dup i 1- bl display
    2dup i ring-char display
  loop
  2drop
;

: moveleft	\ size source.tower destiny.tower --
  pos  swap pos 1- do
    dup i 1+ 1 bl display
    dup i 1 ring-char display
  -1 +loop
  drop
;

: moveright	\ size source.tower destiny.tower --
  pos 1+  swap pos 1+ do
    dup i 1- 1 bl display
    dup i 1 ring-char display
  loop
  drop
;

: traverse	\ size source.tower destiny.tower --
  2dup >
  if  moveleft  else  moveright  then
;

: tmove		\ size source.tower destiny.tower --
  key? if
    key drop
    0 n 4 + at-xy
    ( cursor-on )  200 throw
  then
  rot rot 2dup raise
  >r 2dup r> rot traverse
  2dup ring + 1- c! swap lower
  100 delay
;

: multimov	\ size source destiny spare --
  3 pick 1 =
  if    drop tmove
  else  >r >r swap 1- swap r> r>
        4dup swap recurse
        4dup drop  rot 1+ -rot  tmove
        -rot swap recurse
  then
;

: maketower	\ tower --
  pos  4 n +  3
  do  dup i at-xy  v-bar emit loop
  drop
;

: makebase	\ --
  0 n 4 + at-xy  n 6 * 3 + 0
  do  base-char emit  loop
;

: makering	\ tower size --
  2dup ring + 1- c! swap lower
;

: setup		\ --
  cls  n 1+ 0
  do  1 ring i + c!  loop
  3 0
  do  i maketower  loop
  makebase  1 n
  do  0 i makering  -1 +loop
;

: (towers)	\ #rings --
  3 max  nmax min  to n
  ( cursor-off ) setup  n 2 0 1
  begin
    over pos n 4 + at-xy  n 0
    do   7 emit  25 delay   loop
    rot 4dup multimov
    hell.freezes.over
  until
  ( cursor-on )
;

: towers	\ #rings --
  OpenHanoi
  ['] (towers) catch
  if  drop  then
  CloseHanoi
  cr
;
