\ swk5 "swiss knife" 4bit I/O       2014_10_03
\ Release 0.34 using MPE XC Lite on TI MSP430G2553

\ 20141103 SFP001 Conversion to XC Lite - compiled
\                 Changed names of 0! and 0@ and friends
\                 to avoid name conflicts.
\                 Changed OUT to OUTP2.
\ started: 2014_10_26 now 2014_10_29
\ Issues:
\ Glitches while setting upper nibble of P2 - fixed OUT!
\ port name P2 is in conflict with modul name P2 - renamed.

\ added: 4 servo pulses to OUT (30.10.2014 22:47)
\ renamed demos to EXx (29.10.2014 21:19)
\ added: some demos Px (29.10.2014 13:05)
\ added: ADC  29.10.2014 00:02
\ first test today 26.10.2014 23:49

\   Compiles in target without errors.
\   blocks with mk0 mark = tested ok on first glance ...
\   ... functioning, stack ok.
\   MYBONNIE is playing, no stack error - ok
\   PWM running, SWEEP is ok.
\   I/O and nibbels working on variables.
\   mem u. 6703 (plenty flash left)


\ ********************************************
\  Start of Program:  SWK4 programmed in Forth
\ ********************************************

\ This program uses some of the routines
\ planned for the self-contained Application
\ Starting Programming – with or without using a PC.
\ We use some of these routines here as an example
\ of how to program the MSP430 Controller.
\  Add the 5 resistor LED combinations
\  and the 3 switches to 430 board
\  and run routines completely in the MSP430, using Forth

\ Defining I/O for this application

\ Device Pinout: MSP430G2553 20-Pin PDIP
\ (TOP VIEW)

\ VCC-----------------[01   20]-------------VSS
\ (LED1 LP) AD0 P1.0--[02   19]--P2.6 OUT2
\ RXD-----------P1.1--[03   18]--P2.7 OUT3
\ TXD-----------P1.2--[04   17]--test
\ (S2 LP)----S2 P1.3--[05   16]--RST      S1-LP
\ ..........AD4 P1.4--[06   15]--P1.7 S1          [SPI]       
\ _-_-_-_-_-FRQ P1.5--[07   14]--P1.6 PWM LED2-LP [SPI] TA0.1 
\ ----------IN0 P2.0--[08   13]--P2.5 OUT1
\ ----------IN1 P2.1--[09   12]--P2.4 OUT0
\ ----------IN2 P2.2--[10   11]--P2.3 IN3

\ P1.0 used as Input, later on as Analog Input 1
\ P1.1 RX and
\ P1.2 TX are used as the serial interface to the PC
\ P1.3 used as S2 as on the TI Launchpad,
\      internal resistor enabled
\ P1.4 used as Input, later on as Analog Input 2
\ P1.5 used as Output,
\      later on to output a frequency with defined length
\ P1.6 Outout, later on as Pulse Width Modulation output
\      quasi D/A Output
\ P1.7 Input S1, internal resistor enabled

\ All P2.x Inputs have internal resistor enabled,
\ so open input means HIGH
\ P2.0 Input 0
\ P2.1 Input 1
\ P2.2 Input 2
\ P2.3 Input 3

\ P2.4 Output 0
\ P2.5 Output 1
\ P2.6 Output 2
\ P2.7 Output 3


\ The MSP 430 will be rather fully used.
\ So to save Program and RAM Space, the 4 Bit Nibbles are
\ packed into 4x4 nibbles to be one Variable (16 bit cell).

\ Some scratch variables    mk0
\ IN  4 bit input, when used reflects status of 4 Input lines
\ F   4 bit value for tones
\ A   4 bit A register
\ B   4 bit B register
variable IFAB   0 IFAB !  \ define the first variable with                         \ the first 4 nibbles and init to 0

\ W   4 bit W variable
\ X   4 bit X variable
\ Y   4 bit Y variable
\ Z   4 bit Z variable
variable WXYZ   0 WXYZ !  \ define the second variable with                         \ the second 4 nibbles and init to 0

\ A1  4 bit analog in channel 1
\ A2  4 bit analog in channel 2
\ P   4 bit pulse width modulated output
\ O   4 bit output register
variable 12PO   0 12PO !  \ define the third variable with                         \ the third 4 nibbles and init to 0

\ define a Word to show a cell at an address unsigned   mk0
: ? ( adr -- )   @ u. ;

\ As we have packed these nibbles into 1 variable,
\ some words now for juggling these nibbles
\ basically having this 16 bit variable and store 4 nibbles
\ into the same Variable, e.g. get the nibbles in and out
\ Looks difficult but is easy with some explations for these
\ bits
\  15  14  13  12   11  10   9   8   7  6  5  4    3  2  1  0
\ A13 A12 A11 A10  A23 A22 A21 A20  P3 P2 P1 P0   O3 O2 O1 00
\ Saving bytes in RAM, we must be as effective as possible

hex
\ This HEX tells the computer that we input values in
\ HEX and the computer will output as data as HEX until
\ changed.

: n0! \ n adr -- \ store nibble 0 to variable at address
  >r r@ @ FFF0 and swap F and + r> ! ;

\ n is the nibble to be saved at the right 4 bits within the 16
\ adr is the address where the relevant variable is
\ Nibble 0 of a variable at a defined address
\ will be taken from the stack (all data is normally 16 bit)
\ transferred to the Return Stack
\
\ then ANDed with FFF0, to set the relevant bits to 0
\ add the nibble

: n1! \ n adr -- \ store nibble 1 to variable at address
  >r r@ @ FF0F and swap F and 4 lshift + r> ! ;
  
\ exactly the same, except that 4 left shifts of the nibble
\ are necessary before adding

: n2! \ n adr -- \ store nibble 2 to variable at address
  >r r@ @ F0FF and swap F and 8 lshift + r> ! ;
  
\ exactly the same, except that 8 left shifts of the nibble
\ are necessary before adding

: n3! \ n adr -- \ store nibble 3 to variable at address
  >r r@ @ 0FFF and swap F and C lshift + r> ! ;

\ exactly the same, except that 12 left shifts of the nibble
\ are necessary before adding

\ This Code here is the inverse routine
\ cutting out the relevant nibble of the 16 bit variable

: n0@ \ adr - n \ fetch nibble 0 of variable at address
  @ 000F and ;
\ dealing with the lowest 4 bit, just make the others 0
\ by ANDing the 16 bit variable with 000F

: n1@ \ adr - n \ fetch nibble 1 of variable at address
  @ 00F0 and 0004 rshift ;
\ dealing with bit 7 to 4 requires ANDing with 00F0
\ and shifting them to the right by 4

: n2@ \ adr - n \ fetch nibble 2 of variable at address
  @ 0F00 and 0008 rshift ;
\ dealing with bit 11 to 8 requires ANDing with 0F00
\ and shifting them to the right by 8

: n3@ \ adr - n \ fetch nibble 3 of variable at address
  @ F000 and 000C rshift ;
  
\ dealing with bit 15 to 12 requires ANDing with F000
\ and shifting them to the right by 12


\ ***********************************
\ Now about the Application MultiTool
\ ***********************************
\ As a first step we have to define the relevant IO Bits
\ Either as Input (including enabling internal pull-up
\ resistor to save on external resistors)
\ or as Output.
\ Here you can find the relevant memory mapped addresses
\ to set the functionality:
\ Read Input Data,
\ Write Output Data
\ Some of the functions possible on these IO pins can be
\ rather complex, so in the beginning
\ we will just use them as Input and Output only,
\ For some more advanced functions see separate Forth Code
\ later
  
HEX
: INITIO \ -- \ I/O initialisation of JPS4 ports   mk0
\ mask adr op
\       76543210
[ bin ] 10011001 [ hex ] 0022 ( P1DIR ) bbic! \ P1 INs
[ bin ] 01100110 [ hex ] 0022 ( P1DIR ) bor! \ P1 OUTs
[ bin ] 10001000 [ hex ] 0024 ( P1IES ) bor! \ falling edge detect
[ bin ] 00100000 [ hex ] 0026 ( P1SEL ) bor! \ P1.5 sec func TA0.0 (FRQ)
[ bin ] 10001000 [ hex ] 0027 ( P1REN ) bor! \ pullup selected
[ bin ] 10000000 [ hex ] 0021 ( P1OUT ) bor! \ P1.7 pullup enabled

[ bin ] 00001111 [ hex ] 002A ( P2DIR ) bbic! \ P2 INs
[ bin ] 11110000 [ hex ] 002A ( P2DIR ) bor! \ P2 OUTs
[ bin ] 00001111 [ hex ] 002F ( P2REN ) bor! \ P2 pullups selected
[ bin ] 00001111 [ hex ] 0029 ( P2OUT ) bor! \ P2 pullups enabled
[ bin ] 11000000 [ hex ] 002E ( P2SEL ) bbic! \ clear these bits to I/O
; INITIO


\ Basic I/O          mk0
HEX
: OUT!  \ n -- \ write OUT to P2 upper nibble (4 LEDS)
  04 lshift    \ move to n upper nibble
  0F or        \ keep input pullups set!
  0029 c! ;    \ write port

: OUT>  \ -- \ copy O to OUT!
  12PO n0@ out! ;

: OUTP2  \ n -- \ save n to O and do OUT!
  12PO n0! out> ;

: IN@  \ -- n \ read digital input pins - nibble
  0028 c@ 000F and ;

\ Helper Words to output a nibble to the terminal
HEX

: zero  030 hold  ;

: 4#  \ n --   \ display 4 digits unsigned with leading zeros
  <#  zero # # # #   #> TYPE SPACE ;
: 2#  \ n --   \ display 2 digits unsigned with leading zeros
  <#  zero # #    #> TYPE SPACE ;

\ test falling edge at switch S1 and S2      mk0
023 constant P1IFG
: S1?  080 P1IFG btst ;   \ -- f   \ set on edge event
: S2?  008 P1IFG btst ;   \ -- f   \ set on edge event
: S1-  080 P1IFG bbic! ;   \ --   \ reset edge event flag
: S2-  008 P1IFG bbic! ;   \ --   \ reset edge event flag



\ Delays    mk0
\ 1MS is prefefined in 4e4th. Its an empty loop,
\ predefined for MCU running with 8Mhz DCO = default.
\ Use Forth word MS to make any ms delay.
\ Example:
\ decimal 10 ms   \ delay of 10 miliseconds.

DECIMAL
: 1SEC  \ -- \ delay of one second
  1000 ms ;

: SECS  \ n --   \ delay of n seconds      mk0
  0 DO 1sec LOOP ;



\ ------------------------------------------------------------
\ Utilities for this MCU
\ ------------------------------------------------------------

\ ------------------------------------------------------------
\ Tones
\ Output is for MSP430G2553, 8Mhz DCO and SMCLK /2

HEX
\ Output a square wave at P1.5 using timer TA0.0      mk0

\ Use P1.5 as timer TA0.0 ouput.
\ This is done by setting pin P1.5 to its second function.
\ Then set and start Timer.
\ Note: In this second function, the pin is no longer
\ a general purpose I/O pin. It is switched to another I/O
\ mode called 'second function' of the pin.

: P15SEC   \ set P1.5 to its second function
  020 dup  0022 bor!  026 bor! ;

: P15IO     \ set back P1.5 to its GPIO
  020 026 bbic!  020 041 bbic! ;

: TON-  zero 0160 ! p15io ;  \ stop timer

: TON+ \ n --   \ start timer-A with interval n
  p15sec        \ init pin
  0080  0162 !  \ CCTL0  set timer output mode
  ( n ) 0172 !  \ CCR0   set interval
  0254  0160 !  \ CTL    start timer clock, mode and divider
  ;

\ midi tones B3 to B5, 2 octaves; pitch list.   mk0

DECIMAL
create TONELIST	\ -- addr
  0000 ,  \ no tone
  7962 ,  \ B3
  7515 ,  \ C4
  6695 ,  \ D4
  5965 ,  \ E4
  5630 ,  \ F4
  5016 ,  \ G4
  4469 ,  \ A4
  3981 ,  \ B4
  3758 ,  \ C5
  3348 ,  \ D5
  2983 ,  \ E5
  2815 ,  \ F5
  2508 ,  \ G5
  2235 ,  \ A5
  1991 ,  \ B5

: T@   \ i -- n   \ get pitch using index i
  cells tonelist + @ ;
  
\ Demo song
DECIMAL
: NOTE   \ i --   \ play note i
  t@ ton+ 500 ms ;
: LNOTE
  note 500 ms ;
: PAUSE \ n --   \ play a pause
  ton- ms ;  \ ( use n = 50,100,200)

HEX
\ test melody: tone, delay, pause, delay, ...    mk0
\ Connect speaker between P1.5 and GND.
: mybonnie
  06 note 50 pause   \ G4
  0C note 50 pause   \ F5
  0A note 50 pause   \ D5
  09 note 100 pause   \ C5
  0A note 50 pause   \ D5
  09 note 50 pause   \ C5
  07 note 100 pause   \ A4
  06 note 50 pause   \ G4
  04 note 100 pause   \ E4
  06 note 50 pause   \ G4
  0C note 50 pause   \ F5
  0A note 100 pause   \ D5
  09 note 50 pause   \ C5
  09 note 50 pause   \ C5
  08 note 100 pause   \ B4
  09 note 50 pause   \ C5
  0A lnote 200 pause   \ D5
  ton-  ;



\ ------------------------------------------------------------
\ PWM of 16KHz                      mk0
\ for 8MHZ DCO using timer TA0

HEX
: P16IO   \ --   \ set P1.6 to GPIO
  0040 0022 bor!    \ P1DIR   P1.6 OUT
  0040 0026 bbic!    \ P1SEL   P1.6 GPIO
  ;

: P16SEC  \ --   \ set P1.6 to second function
  0040 0022 bor!    \ P1DIR   P1.6 OUT
  0040 0026 bor!    \ P1SEL   P1.6 select second function
  ;

: PWM-
  zero 160 !  \ TA0CTL   stop timer
  p16io 0040 0021 bbic! ; \ set p1.6 I/O and clear p1.6

: PWM+  \ n --  \ init and start PWM at P1.6
  01F4 0172 !   \ TA0CCR0    set period 16KHz at 8MHZ DCO
  00E0 0164 !   \ TA0CCTL1   set output mode
       0174 !   \ TA0CCR1    set pulsewidth
  0210 0160 !   \ TA0CTL     set timer mode and run
  ;

: PWM  \ n --   \ set PWM at P1.6   n = 0...F
  ton- 000F and
  dup 0= IF drop pwm-
  ELSE 001F * pwm+ p16sec THEN ;



\ ------------------------------------------------------------
\ ADC
HEX
\ You may name addresses and bits.

\ address name \ function
\ 01B0 constant ADC10CTL0 \ ADC10 control register 0
\ 01B2 constant ADC10CTL1 \ ADC10 control register 1
\ 004A constant ADC10AE0  \ ADC10 input enable register 0
\ 01B4 constant ADC10MEM  \ ADC10 Memory, conversion result

\ some named bits
\ 0002 constant ENC       \ ADC10 Enable Conversion bit
\ 4000 constant INCH_4    \ Selects Channel 4
\   10 constant BIT4      \ for P1.4 Analog Input Enable
\ 0001 constant ADC10SC   \ start conversion bit
\ 0004 constant ADC10IFG  \ ADC10 Interrupt Flag

\ option bits are:
\ 2000 constant SREF_1     \ set VR+ = VREF+ and VR- = AVSS
\ 1000 constant ADC10SHT_2 \ 16 x ADC10CLKs
\ 0040 constant REF2_5V    \ ADC10 Ref 0:1.5V / 1:2.5V
\ 0020 constant REFON      \ ADC10 Reference on
\ 0010 constant ADC10ON    \ ADC10 On/Enable
\ ----
\ 3070 <-- sum of option bits

: ADCOFF   \ --  \ stop ADC10 ...
  zero 01B0 ! ;   \ ... can be modified only when ENC = 0

: ADCON    \ --  \ start ADC10
  0002 01B0 or! ;  \ Set ENC

: ADC@   \  -- x   \ get ADC value
  03 01B0 or! \ start conversion (ENC+ADC10SC bits)
  BEGIN 04 01B0 btst UNTIL \ test BIT2 (ADC10IFG)
  01B4 @  ; \ get result

: ADC4     \ --   \ select and init ADC channel 4
  adcoff
  3070 01B0 !      \ set options (see MCU user manual)
  4000 01B2 !      \ select input channel (A4 at P1.4)
  10 004A bor! ;   \ set pin, analog input enable

\ Example:
\ adc4 adcon adc@ .  \ print single conversion



\ ------------------------------------------------------------
\ 4 servos using the same 4 OUT pins
HEX
\ load Xn with values 164...328 - 164 positions per servo line
variable x0
variable x1
variable x2
variable x3

: LDX \ x0 x1 x2 x3 -- \ load presets to variables X0 .. X3
  x3 ! x2 ! x1 ! x0 ! ;

: DECN \ x -- \ loop decrementing x until zero
  BEGIN dup 0= invert WHILE 1- REPEAT drop ;

: PULS \ x n -- \ send puls x to line n ; x=80,40,20,10
  >r r@ 029 bor! decn r> 029 bbic! 4 ms ;

: SERVO \ --- \
  initio
  BEGIN
  x0 @ 10 puls
  x1 @ 20 puls
  x2 @ 40 puls
  x3 @ 80 puls
  key? UNTIL
  key drop ;

\ test: decimal 200 200 200 200 ldx servo



\ ------------------------------------------------------------
\ Example Programms
HEX
: EX0  \ -- \  display ADC at OUT and play note      mk0
  initio adc4 adcon  \ init all modules
  BEGIN
  adc@ 44 /          \ scale 10Bit value, 3FF..0 --> F...0
  dup outp2          \ display scaled value
      note 50 pause  \ play apropriate tone
  key? UNTIL         \ leave loop, clean up
  key drop zero outp2  ton- ;

HEX
: EX1  \ -- \  IN to OUT                             mk0
  initio
  BEGIN
  in@ outp2             \ get IN and store it to OUT
  key? UNTIL key drop ; \ leave loop, clean up

HEX
: EX2 \ -- \   IN to PWM and OUT                     mk0
  initio
  BEGIN
  in@                \ get IN
  dup outp2          \ store to OUT
      PWM  1sec      \ do PWM for a second
  key? UNTIL         \ leave loop, clean up
  key drop zero outp2 pwm- ;

HEX
: EX3  \ -- \   faling edge detector S2 to OUT0      mk0
  initio s2-         \ init system and reset edge detection
  BEGIN
  s2? IF
      1 outp2              \ set on edge detect
      1sec s2- zero outp2  \ wait, then reset
      THEN
  key? UNTIL         \ leave loop, clean up
  key drop zero outp2 ;

HEX
: EX4  \ -- \   falling edge detector S2 to note      mk0
  initio s2-  \ init system and reset edge detection
  01 note     \ start low note
  BEGIN
  s2? IF
      1 outp2              \ set on edge detect
      0F note              \ do high note
      1sec s2- zero outp2  \ wait, then reset
      01 note
      THEN
  key? UNTIL         \ leave loop, clean up
  key drop zero outp2 ton- ;

HEX
: tout \ c -- c \
  dup [char] 1 = IF 01 12po btoggle! out> exit THEN
  dup [char] 2 = IF 02 12po btoggle! out> exit  THEN
  dup [char] 3 = IF 04 12po btoggle! out> exit  THEN
  dup [char] 4 = IF 08 12po btoggle! out> exit  THEN
  dup [char] 0 = IF zero outp2 exit THEN
  dup [char] f = IF 0f outp2 exit  THEN  ;

: EX5 \ -- \  press 1 2 3 4 to toggel OUT bits,
          \ 0 to clear all, F to set all.
  initio zero outp2
  BEGIN
  key tout
  1B ( esc ) = UNTIL zero outp2
  ; \ exit on esc-character


\ P6,7 tbd

HEX
: EX8 \ -- \ ADC to OUT and terminal                 mk0
  initio adc4 adcon  \ init all modules
  BEGIN
  adc@ 44 /          \ scale 10Bit value, 3FF..0 --> F...0
  dup outp2          \ display scaled value
  dup note 50 pause  \ play apropriate tone
      .              \ print to terminal
  key? UNTIL         \ leave loop, clean up
  key drop zero outp2  ton- ;

\ EX9... tbd

: EXD \ -- \   terminal KEY to OUT with echo          MK=
  initio zero outp2
  cr ." press any key to start - press Esc-key to exit"
  cr  \ new line
  BEGIN
  key       \ get key
  dup over 20 < IF
    5E emit 40 + emit   \ echo control character
    ELSE emit THEN      \ echo character
  dup space 2# space    \ print character value
  dup outp2             \ send lower nibble to OUT
  1B ( esc ) = UNTIL
  ; \ exit on esc-character

\ EXE ... EXF tbd

\ --> swk4-04.txt

\ save

\ ---------------
\ todo ***
\ P0..PF loop

decimal

.free




