\ 68HC12 ANS Local variables extensions

((
        Copyright (c) 1998-1999
        MicroProcessor Engineering
        133 Hill Lane
        Southampton SO15 5AF
        England

        tel: +44 (0)2380 631441
        fax: +44 (0)2380 339691
        net: mpe@mpeforth.com
             tech-support@mpeforth.com
        web: www.mpeforth.com

By comparison with other processors, this CPU is too register limited
to provide efficient use of local variables as there is no register
in which to keep a local pointer. Keeping the LP (local pointer) as a
USER variable simply consumes another index register or two as the UP
(user area pointer) is itself in a variable.

A better solution is to keep the LP in a variable of its own, but
this would requires saving and restoring LP in both interrupt routines
and in context switches. However, from the code below this doubles the
speed of inline code generation for LV access. Whether this is woth it
given the speed of MAKELVs is dubious, but having LP as a variable speeds
up MAKELVS as well, which makes the whole effort worthwhile.

20000102 SFP001 Fixed MAKELVS
19990607 SFP000 Initial release with local pointer in the variable LP
))

only forth definitions
hex

\ ------------------------------------------------------------------------------
\       runtime primitives
\ ------------------------------------------------------------------------------

internal

optimised? 0= [if]
CODE LV!        \ n -- ; store local variable [offset in-line]
  tfr  d, y                             \ save TOS
  ldd  lp                               \ Locals pointer
  addd 0 [,sp]                          \ add offset
  exg  d, y                             \ D=TOS, Y=addr[n]
  std  0 ,y                             \ stash
  ldd  2 ,x+                            \ restore TOS
  puly                                  \ return address
  jmp  2 ,y                             \ step over inline address
END-CODE

CODE LV@        \ -- n ; read local variable [offset in-line]
  std  2 ,-x                            \ save TOS
  ldd  lp                               \ locals pointer
  addd 0 [,sp]                          \ add offset
  tfr  d, y                             \ Y=addr[n]
  ldd  0 ,y                             \ fetch from local
  puly                                  \ return address
  jmp  2 ,y                             \ step over inline address
END-CODE

CODE LV         \ -- a-addr ; read local variable address [offset in-line]
  std  2 ,-x                            \ save TOS
  ldd  lp                               \ Locals pointer
  puly                                  \ return address
  addd 2 ,y+                            \ add offset, step over inline address
  jmp  0 ,y                             \ return
END-CODE
[then]

\ frame for makelvs
0 equ l_ra              \ saved return address
2 equ l_oldrsp          \ saved RSP
4 equ l_newpsp          \ new PSP value
6 equ l_framesize       \ size of this frame

CODE MAKELVS    \ -- ; make space for local variables ; LP is a variable
                \ #args (cells), #locals (cells) in-line
                \ Frame structure is as OTA spec, vol1 3.2.5
                \ arguments +ve offset, locals -ve offset

  std  2 ,-x                            \ stash TOS - all args now on stack
  leax l_framesize negate ,x            \ build frame
  puly   sty  l_ra ,x                   \ save return address - USED LATER
  sts  l_oldrsp ,x                      \ and stack pointer

\ allocate a frame for the arguments
  ldab 0 ,y                             \ get #args (cells)
  ne, if,
\    clra  asld                          \ convert #args to bytes
\    coma   comb   addd # 1              \ and make negative
\    leas d,sp                           \ drop frame
    aslb				\ bytes > cells
    negb				\ negate
    sex  b, d				\ sign extend
    leas d,sp                           \ drop frame
  endif,
  
\ save data so that we can unwind the local frame later
  movw l_oldrsp ,x  2 ,-sp		\ save old RSP
  movw $ext lp  2 ,-sp			\ save old LP
  sts  lp                               \ set new LP
  
\ copy the arguments into the frame
  ldab 0 ,y                             \ get #args again
  leay l_framesize ,x                   \ point to first arg - DOES NOT SET FLAGS!
  ne, if,                               \ if arguments to copy
    pshx                                \ save PSP/framepointer
    leax 6 ,sp                          \ point to first location in arg frame	SFP001
    begin,                              \ transfer arguments
      movw  2 ,y+  2 ,x+
      decb
    eq, until,
    pulx                                \ restore PSP/frame
  endif,    
  sty  l_newpsp ,x

\ reserve local variable space
  ldy  l_ra ,x   ldab 1 ,y              \ get #temps(cells)
  ne, if,
\    clra   asld                         \ convert #args to bytes
\    coma   comb   addd # 1              \ and make negative
\    leas d,sp                           \ drop frame
    aslb				\ bytes > cells
    negb				\ negate
    sex  b, d				\ sign extend
    leas d,sp                           \ drop frame
  endif,

\ remove the scratch frame and the arguments, and get out
  ldx  l_newpsp ,x			\ restore PSP
  ldd  2 ,x+				\ restore TOS
  jmp  2 ,y
END-CODE

optimised? 0= [if]
CODE RELLVS     \ -- ; drop local variables
  ldy  lp                               \ get current LP
  movw 2 ,y+  $ext lp                   \ restore frame pointer
  ldy  0 ,y                             \ get return stack value back
\ N.B we must avoid interrupt problems caused by setting the stack pointer
\ before the return address has been put somewhere safe, so we transfer it
\ from the frame to the return stack we're restoring before SP is set. This
\ keeps the system interrupt safe as SP is the last thing changed. This may
\ be picky, but this has caused interrupt problems before!
  movw 2 ,sp+  -2 ,y                    \ copy return address to old return stack
  tfr  y, sp                            \ y is old SP
  rts
END-CODE
[then]

external

