\ IntroLite.man

((
Copyright (c) 2013, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

Change History
==============
))

\ ************
\ *! introlite
\ *T Introduction
\ ************

\ *P This manual documents the MPE Forth kernel for the MSP430
\ ** Launchpad with an MSP430G2553 processor.


\ *******************
\ *S Before you start
\ *******************
\ *P The MPE Forth kernel uses the hardware UART for serial
\ ** communications. This is run through the USB facilities
\ ** on the Launchpad. Make sure that the five links of J3
\ ** and the two links of J5 on the Launchpad are set as below.
\ ** *\image{"Launchpad links" "LaunchpadLinks" "jpg" 75 30}

\ *P Check that the TI DLLs, *\i{MSP430.dll} and *\i{HIL.dll}
\ ** are installed where the MPE cross-compiler can find them.
\ ** Installation is dicussed in chapter 4, "JTAG and and the MSP430"
\ ** of the MSP40 cross-compiler manual, *\i{Docs/MSP430man.pdf}.
\ ** If the DLLs are incorrectly installed, you will not be able
\ ** to program the Launchpad with the Forth kernel.


\ ********************
\ *S Working with AIDE
\ ********************
\ *P It seems to be essential that every language has an IDE. If
\ ** you do not like IDEs, you can skip this section. AIDE has
\ ** three parts:
\ *(
\ *B Editor - ForthEd2 is a trivial editor just so that AIDE has
\ ** one. There are many far better programming editors. You can
\ ** get AIDE to use a different one using the
\ ** *\i{IDE -> Configure/Locate} menu entry. See the AIDE manual
\ ** for more details.
\ *B Tool Capture Display - the cross compiler window in most cases.
\ ** Note that this is software running on the host PC. For a standalone
\ ** Forth target there is no connection to the target, except for
\ ** Flash programming and all target commands must be entered in
\ ** a terminal emulator, usually AIDE's PowerTerm. For an
\ ** Umbilical Forth target, you can enter target commands and the
\ ** cross compiler will try to execute them on the target.
\ *B PowerTerm - a terminal emulator used with standalone Forth
\ ** systems. It is specially adapted for debugging complex targets
\ ** and features up to eight individually controllable
\ ** cursor-addressable displays - ideal for debugging multi-tasking
\ ** targets.
\ *)


\ ***********************
\ *S Producing the kernel
\ ***********************
\ *P The Forth kernel is built using a cross compiler running
\ ** in the AIDE environment. AIDE contains three primary windows:
\ ** compiler, terminal emulator and text editor. The compilation
\ ** results can be seen in the compiler window. During compilation,
\ ** the compilation results can be seen in the compiler/tool window.

\ *P To compile the standalone Forth kernel, find the "liteLP2553sa"
\ ** button in the main toolbar and click it to compile from the
\ ** control file *\i{liteLP2553sa.ctl}.
\ ** The compiler will ask you whether you want to program the
\ ** Launchpad with the new kernel. Answer 'Y' to the download and
\ ** erase prompts.

\ *P After programming the Launchpad, the new kernel will be running.
\ ** To talk to the kernel, use AIDE's PowerTerm terminal emulator.
\ ** The kernel talks to PowerTerm through the Launchpad's application
\ ** UART. Finding this UART can require a little botheration. There
\ ** are two ways to do this.
\ *( 1
\ *B Go to Windows Control Panel -> System -> Device Manager -> Ports.
\ ** One of the entries will be something like:
\ ** *\b{MSP430 Application UART (COM10)}.
\ ** COM10 will be the UART you will use.
\ *B Go to AIDE's PowerTerm configuration dialog. Select the drop
\ ** down list in the COM Port# group. It will list all the available
\ ** COM ports by COM number and name. The one you want includes:
\ ** *\b{COMxx MSP430 Application UART}.
\ *)
\ *P Now select the required port in the PowerTerm configuration
\ ** dialog and ensure that it is set to 9600 baud, 8 data bits, 1
\ ** stop bit, no parity and file server enabled. Save the
\ ** configuration and press OK.
\ ** Start the PowerTerm connection and you should be talking to
\ ** the kernel.

\ ******************************
\ *S About the standalone kernel
\ ******************************
\ *P The control file *\i{liteLP2553sa.ctl} specifies the use of
\ ** the target Flash as follows:
\ *E  Kernel: $D000..FFE0, then vectors to $FFFF
\ **  App:    $C000..CFFF
\ *P The App space is where the target kernel compiles code.

\ *P The Forth kernel you have made and installed is a cut-down
\ ** version of the full MPE Forth kernel. It has extensions so
\ ** that code is compiled directly into the MSP430 Flash. When
\ ** you want to add code, you can compile it directly on the
\ ** Forth kernel by using the phrase
\ *C   include <filename>
\ *P on the target Forth command line. AIDE will then try to
\ ** deliver the file to the kernel. If the file cannot be
\ ** found, AIDE will let you change the default directory
\ ** (the commonest problem) or correct the spelling.

\ *P To start over just type *\fo{EMPTY}, the application
\ ** region of the Flash will be cleaned, and the CPU is
\ ** rebooted. To reuse your compiled code at the next reboot
\ ** or power up, use the word *\fo{COMMIT}.
\ ** If you just want to preserve the code, use:
\ *C   0 COMMIT
\ *P If you want the Forth to run a word, say *\fo{APP}, use:
\ *C   ' APP COMMIT


\ ************************
\ *S About Umbilical Forth
\ ************************
\ *P The control file *\i{liteLP2553uf.ctl} specifies the use of
\ ** an Umbilical Forth system in which the cross compiler provides
\ ** all the interactivity. Because the TI Launchpad uses a *\b{very}
\ ** slow USB connection, the interaction takes a very long time.
\ ** It was designed for use with a 115200 baud serial line.

\ *P Umbilical Forth provides a minimal Forth system on the target
\ ** for situations where every byte matters. You can put far more
\ ** functionality onto a 16 kb Umbilical Forth system than on a
\ ** 16 kb standalone Forth.


\ **********
\ *S Gotchas
\ **********

\ =================
\ *N Flash problems
\ =================
\ *P If you forget to use *\fo{COMMIT} and *\fo{EMPTY} appropriately,
\ ** the MSP430 Flash may not be correctly erased and so application
\ ** compilation may fail. Try to use *\fo{EMPTY} and if that fails,
\ ** reinstall a new kernel.

\ *P The Flash programming DLLs are unreliable on many PCs. Just
\ ** accept this as the cost of cheap hardware and software.
\ ** Eventually, the Flash will be programmed, but you may need
\ ** to recompile a few times to achieve this.

\ ==============
\ N DLL problems
\ ==============
\ *P The Launchpad is built down to a price. The JTAG facilities
\ ** are provided by a secondary CPU on the Launchpad and by
\ ** Windows DLLs. In the past this combination has shown a
\ ** few problems, especially if the Launchpad has been unexpectedly
\ ** disconnected and reconnected.

\ *P If you come across such a problem, shut down all the software
\ ** that could be using the MSP430 DLLs. Then restart the software.
\ ** If problems persist, disconnect the Launchpad, reboot the PC,
\ ** reconnect the Launchpad, and only then restart the software.

\ ================
\ *N Serial lockup
\ ================
\ *P Occasionally the connection to the USB UART refuses to open
\ ** properly and AIDE's PowerTerm appears to lock up. DO NOT CLOSE
\ ** THE CONNECTION. Instead, remove the USB cable and wait for
\ ** PowerTerm to detect this and close the connection. Then
\ ** reconnect the USB cable and attempt to open the UART again.
\ ** You may have to do this a few times to establish a good
\ ** connection.


\ ******
\ *> ###
\ ******

