\ XC6 Umbilical Forth minimum code file for MSP430

((
Copyright (c) 2002, 2007, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============

Target Model:
=============
Forth           MSP430          Comments
----------------------------------------
IP		R0/PC		MSP430 PC
RSP             R1/SP           MSP430 return stack
--		R2/CG1/SR
--		R3/CG2
PSP             R4
TOS             R5
UP              R6		USER area pointer
LP (locals)     R7		points to LOCAL frame on return stack
scratch		R8..R13		with R5 forms working stack
codegen		R14		code generator temporary
codegen		R15		preserves SR in shuffle
----------------------------------------
))

decimal

\ =========
\ *! min430
\ *T Minimal Umbilical code definitions
\ =========
\ *P The file *\i{Min430.fth} contains the minimum code definitions
\ ** required to support Umbilical Forth. If additional definitions
\ ** are required, they may be copied to a new file from *\i{Code430lite.fth}
\ ** or *\i{kernel72lite.fth}.

\ *P Note that many of the words documented here are only included
\ ** by the early non-optimising compiler. These words will be
\ ** removed from this file in a future release. If needed, they
\ ** may then be copied from another file.


\ ********************************
\ *S Logical and relational operators
\ ********************************

((
unoptimised? [if]
: min		\ n1 n2 -- min(n1,n2)
\ *G Given two data stack items preserve only the smaller.
  2dup >
  if  nip exit  then
  drop
;

: max		\ n1 n2 -- max(n1,n2)
\ *G Given two data stack items preserve only the largerr.
  2dup <
  if  nip exit  then
  drop
;
[then]
))

code within?    \ x a b -- t/f ; true if a<=x<=b
\ *G Return TRUE if N1 is within the range N2..N3.
\ ** This word uses signed arithmetic.
  cmp	0 (psp)  2 (psp)
  jl    witf
  cmp   2 (psp)  tos
  jl    witf
  mov	# -1  tos		        \ true exit
  jmp   witx
l: witf
  mov   # 0  tos			\ false exit
l: witx
  add	# 4 psp
  ret
end-code

code within	\ n1|u1 n2|u2 n3|u3 -- flag ; ANS 6.2.2440
\ *G The ANS version of *\fo{WITHIN?}.
\ ** Return TRUE if N1 is within the range N2..N3-1.
\ ** This word uses unsigned arithmetic, so that signed compares are
\ ** treated as existing on a number circle.
  mov  @psp+  r15
  mov  @psp+  r14			\ tos=b, r15=a, r14=x
  sub  r15, r14				\ x:=x-a
  sub  r15, tos				\ b:=b-a
  sub  r14, tos				\ b-x
  sbb  tos  tos				\ set flag
  ret
end-code


\ ************
\ *S Control flow
\ ************

fast-do? 0= [if]
CODE (DO)       \ limit start --
\ *G The run time action of *\fo{DO} compiled on the target.
l: pdo1
  pop   r15			\ return address
  push  @r15+			\ push loop exit address
  mov   @psp+  r14		\ push limit xor $8000
  xor   # $8000  r14
  push  r14
  sub   r14 tos			\ push start-(limit xor $8000)
  push  tos
  mov   @psp+ tos		\ restore tos
  br    r15
end-code

CODE (?DO)      \ limit start --
\ *G The run time action of *\fo{?DO} compiled on the target.
  cmp  @psp tos			\ ok if not same
  jne  pdo1
  add  # 2 psp
  mov  @psp+ tos
  ret
end-code
[then]

\ LOOP +LOOP I J are code generated
((
optimised? 0= [if]
CODE (LOOP)     \ -- ; absolute address follows inline
\ *G The run time action of *\fo{LOOP} compiled on the target.
  pop  r15			\ return address
  inc  0 (rsp)			\ update
  bit  # $0100 sr		\ test overflow bit
  jnz  ploop1			\ done if set
  br   0 (r15)			\ go round again
l: ploop1
  incd r15			\ step over branch address
  add  # 6 rsp			\ remove loop parameters
  br   r15			\ return
end-code

CODE (+LOOP)    \ n --
\ *G The run time action of *\fo{+LOOP} compiled on the target.
  pop  r15			\ return address
  add  tos  0 (rsp)		\ update
  bit  # $0100 sr		\ test overflow bit
  jnz  ploop1			\ done if set
  mov  @psp+ tos		\ restore tos
  br   0 (r15)			\ go round again
l: ploop1
  incd r15			\ step over branch address
  add  # 6 rsp			\ remove loop parameters
  mov  @psp+ tos		\ restore tos
  br   r15			\ return
end-code

code i          \ -- n ; return DO ... LOOP index
\ *G Return the current *\fo{DO ... LOOP} index.
  sub  # 2 psp
  mov  tos 0 (psp)
  mov  2 (rsp) tos
  add  4 (rsp) tos
  ret
end-code

code j		\ -- n ; return DO ... LOOP index
\ *G Return the outer *\fo{DO ... LOOP} index.
  sub  # 2 psp
  mov  tos 0 (psp)
  mov  2 (rsp) tos
  add  4 (rsp) tos
  ret
end-code
[then]
))

(( now has code generator
CODE EXECUTE    \ xt --
\ *G Execute the code described by the XT. This is a Forth equivalent
\ ** to an assembler JSR/CALL instruction.
  mov  tos r15
  mov  @psp+ tos		\ pop TOS
  br   r15			\ go do it
END-CODE
))


\ ****************
\ stack arithmetic
\ ****************

Code Noop	\ -- ; used by multi-tasker
\ *G A NOOP, null instruction.                                          )
  ret
End-Code

CODE S>D        \ n -- d
\ *G Convert a single number to a double one.
  sub  # 2 psp
  mov  tos  0 (psp)
  swpb tos			\ bit 15 -> bit 7
  sxt  tos
  swpb tos
  sxt tos
  ret
end-code

multiplier? [if]

code um*        \ u1 u2 -- ud ; unsigned multiply
  mov  tos  & mpy
  mov  @psp & op2
  mov  & reslo @psp
  mov  & reshi tos
  ret
end-code

code *		\ n1 n2 -- n1*n2 ; signed multiply, : * um* drop  ;
  mov  tos  & mpys
  mov  @psp+ & op2
  mov  & reslo tos
  ret
end-code

code m*		\ n1 n2 -- d ; signed multiply
  mov  tos  & mpys
  mov  @psp & op2
  mov  & reslo @psp
  mov  & reshi tos
  ret
end-code

[else]

proc umul16*16	\ r12:r13 = r14*r15, destroys r11
  mov  # #16  r11		\ counter
  mov  # 0 r12			\ initialise result
  mov  # 0 r13
  begin,
    add  r13 r13  addc r12 r12	\ shift result
    add  r14 r14		\ test r14 top bit
    cy, if,
      add  r15 r13  addc  # 0 r12
    then,
    dec  r11
  z, until,
  ret
end-code

code um*        \ u1 u2 -- ud ; unsigned multiply
\ *G Perform unsigned-multiply between two numbers and return double
\ ** result.
  mov   tos  r14		\ r14=u2
  mov   @psp  r15		\ r15=u1
  call  # umul16*16		\ result in R12:R13
  mov   r12  tos
  mov   r13  0 (psp)
  ret
end-code

code *		\ n1 n2 -- n1*n2 ; signed multiply, : * um* drop  ;
\ *G Standard signed multiply. N3 = n1 * n2.
  mov   tos  r14		\ r14=u2
  mov   @psp+ r15		\ r15=u1
  call  # umul16*16		\ result in R12:R13
  mov   r13  tos
  ret
end-code

code m*		\ n1 n2 -- d ; signed multiply
\ *G Signed multiply yielding double result.
  mov   tos  r14		\ r14=n2
  mov   @psp  r15		\ r15=n1
  mov   r14  r10		\ form sign of result
  xor   r15  r10
  cmp   # 0  r14		\ make n2 positive
  lt, if,  inv  r14   inc  r14  then,
  cmp   # 0  r15		\ make n1 positive
  lt, if,  inv  r15   inc  r15  then,
  call  # umul16*16		\ result in R12:R13
  cmp   # 0 r10			\ test sign of result
  lt, if,
    inv  r12   inv r13		\ negate by invert and add 1
    add  # 1 r13   addc  # 0 r12
  endif,
  mov   r12  tos
  mov   r13  0 (psp)
  ret
end-code

[then]

((
proc udiv32/16	\ r12:r13 / r14 ; quot=r13, rem=r12 ; destroys r11 r15
  cmp  r12 r14			\ check for overflow
  jlo  udiv-oflo
  jeq  udiv-oflo

  mov  # #16 r11
  begin,
    add  r13 r13   addc r12 r12	\ shift dividend
    mov  sr r15			\ preserve carry
    sub  r14 r12		\ trial subtract
    cy, if,			\ carry = NOT borrow
      bic  # 1 r13		\ NB, ok
    else,
      mov  r15 sr		\ carry = old bit 31
      cy, if,
        bic  # 1 r13		\ set, so ok
      else,
        add  r14 r12		\ not set, undo subtraction
      endif,
    endif,
    dec  r11			\ for all 16 bits
  z, until,
  ret				\ quot=r13, rem=r12

l: udiv-oflo
  mov  # $FFFF r13		\ quot
  mov  # 0 r12			\ rem
  ret
end-code
))

proc udiv32/16	\ r12:r13 / r14 ; quot=r13, rem=r12 ; destroys r11 r15
  cmp  r12 r14			\ check for overflow
  jlo  udiv-oflo
  jeq  udiv-oflo

  clr   r15           \ clear result
  mov   # #17 r11
l: div1
  cmp   r14 r12
  jlo   div2
  sub   r14 r12
l: div2
  rlc   r15
  jc    div4
  dec   r11
  jz    div3
  rla   r13
  rlc   r12
  jnc   div1
  sub   r14 r12
  jmp   div2
l: div3
  clrc
l: div4
  mov   r15 r13
  ret

l: udiv-oflo
  mov  # $FFFF r13		\ quot
  mov  # 0 r12			\ rem
  ret
end-code

code um/mod     \ u32 u16 -- urem uquot
\ *G Perform unsigned division of double number UD by single number U
\ ** and return remainder and quotient.
  mov  tos r14			\ u16
  mov  @psp+ r12		\ u32h
  mov  @psp  r13		\ u32l
  call # udiv32/16		\ quot=r13, rem=r12
  mov  r12 0 (psp)		\ rem
  mov  r13 tos			\ quot
  ret
end-code

code sm/rem      \ d n -- rem quot ; symmetric division
\ *G Perform a signed division of double number D1 by single number N2
\ ** and return remainder and quotient using symmetric (normal) division.
  mov  tos r14			\ n16
  mov  @psp+ r12		\ d32h
  mov  @psp  r13		\ d32l
  mov  r14 r10			\ form sign of quotient
  xor  r12 r10
  mov  r12 r9			\ sign of dividend/remainder
  cmp  # 0 r12			\ make dividend positive
  lt, if,
    inv  r12   inv r13		\ negate by invert and add 1
    add  # 1 r13   addc  # 0 r12
  endif,
  cmp  # 0 r14			\ make divisor positive
  lt, if,
    inv  r14    inc  r14
  endif,
  call # udiv32/16		\ quot=r13, rem=r12
  cmp  # 0 r9			\ remainder takes sign of dividend
  lt, if,  inv  r12  inc  r12  endif,
  cmp  # 0 r10			\ set sign of quotient
  lt, if,  inv  r13  inc  r13  endif,
  mov  r12 0 (psp)		\ rem
  mov  r13 tos			\ quot
  ret
end-code

code fm/mod     \ d n -- frem fquot ; floored division
\ *G Perform a signed division of double number D1 by single number N2
\ ** and return remainder and quotient using floored division. See the
\ ** ANS Forth specification for more details of floored division.
  mov  tos r14			\ n16
  mov  @psp+ r12		\ d32h
  mov  @psp  r13		\ d32l
  mov  r14 r10			\ form sign of quotient
  xor  r12 r10
  mov  r14 r9			\ sign of divisor
  cmp  # 0 r12			\ make dividend positive
  lt, if,
    inv  r12   inv r13		\ negate by invert and add 1
    add  # 1 r13   addc  # 0 r12
  endif,
  cmp  # 0 r14			\ make divisor positive
  lt, if,
    inv  r14    inc  r14
  endif,
  call # udiv32/16		\ quot=r13, rem=r12
  cmp  # 0 r9			\ remainder takes sign of divisor
  lt, if,  inv  r12  inc  r12  endif,
  cmp  # 0 r10			\ set sign of quotient
  lt, if,
    inv  r13  inc  r13
    cmp  # 0 r12		\ remainder non-zero
    nz, if,
      dec  r13			\ adjust quotient
      sub  r12 r9   mov  r9 r12	\ rem := divisor - rem
    endif,
  endif,
  mov  r12 0 (psp)		\ rem
  mov  r13 tos			\ quot
  ret
end-code

: /mod          \ n1 n2 -- rem quot
\ *G Signed division of N1 by N2 single-precision yielding remainder
\ ** and quotient.
  >r s>d r> sm/rem
;

: /		\ n1 n2 -- quot
\ *G Standard signed division operator. n3 = n1/n2.
  >r s>d r> sm/rem nip
;

: mod           \ n1 n2 -- rem
\ *G Return remainder of division of N1 by N2. n3 = n1 mod n2.
  /mod drop
;

: MU/MOD	\ ud u -- urem udquot
\ *G Perform an unsigned divide of a double by a single, returning
\ ** a single remainder and a double quotient.
  >r  0  r@ um/mod  r> swap >r  um/mod  r>
;

: */MOD		\ n1 n2 n3 -- rem quot
\ *G Multiply n1 by n2 to give a double precision result, and then
\ ** divide it by n3 returning the remainder and quotient. The point
\ ** of this operation is to avoid loss of precision.
  >r m* r> sm/rem
;

: */		\ n1 n2 n3 -- quot
\ *G Multiply n1 by n2 to give a double precision result, and then
\ ** divide it by n3 returning the quotient. The point
\ ** of this operation is to avoid loss of precision.
  */mod nip
;

: M/		\ d n -- quot
\ *G Signed divide of a double by a single integer.
  sm/rem nip
;

((
optimised? 0= [if]
CODE negate     \ n1 -- -n1
\ *G Negate a single number.
  inv  tos
  inc  tos
  ret
end-code
[then]
))

: ?negate       \ n1 t/f -- n1/-n1
\ *G If flag is negative, then negate n1.
  0< if negate then
;

((
optimised? 0= [if]
: abs		\ n1 --  |n1|
\ *G If n is negative, return its positive equivalent (absolute value).
  dup 0< if negate then
;
[then]
))

code dnegate	\ d1 -- -d1
\ *G Negate a double number.
  inv  tos   inv 0 (psp)		\ negate by invert and add 1
  add  # 1 0 (psp)   addc  # 0 tos
  ret
end-code

: ?dnegate      \ d1 t/f -- d1/-d1
\ *G If flag is negative, then negate d1.
  0< if dnegate then
;

((
: dabs		\ d -- |d|
  dup 0< if  dnegate  then
;
))


CODE D+         \ d1 d2 -- d3
\ *G Add two double precision integers.
  add  @psp+  2 (psp)
  addc @psp+  tos
  ret
end-code

CODE D-         \ d1 d2 -- d1-d2
\ *G Subtract two double precision integers. D3=D1-D2.
  sub  @psp+ 2 (psp)
  subc tos   0 (psp)
  mov  @psp+  tos
  ret
end-code


\ *********************
\ *S Stack manipulation
\ *********************
\ *P Many of the standard Forth stack manipulation words are just
\ ** code generators in the cross compiler and have no target
\ ** versions.

: ROLL          \ n1 n2 .. nk n -- wierd
\ *G Rotate the order of the top N stack items by one place such that
\ ** the current top of stack becomes the second item and the Nth item
\ ** becomes TOS. See also *\fo{ROT}. N.B. Very slow.
  >R R@ PICK
  SP@ DUP 2+   R> 1+ 2* CMOVE>  DROP
;


\ ***********
\ Comparisons
\ ***********
\ *P The standard Forth comparison words words are just
\ ** code generators in the cross compiler and have no target
\ ** versions.


\ ****************
\ memory operators
\ ****************
\ *P Some of the standard Forth memory manipulation words are just
\ ** code generators in the cross compiler and have no target
\ ** versions.

code c@		\ addr -- b
\ *G Fetch and 0 extend the character at memory ADDR and return.
  mov .b  0 (tos) tos
  sxt     tos
  ret
end-code

code @		\ addr -- n
\ *G Fetch and return the CELL at memory ADDR.
  mov  0 (tos) tos
  ret
end-code

code 2@		\ addr -- d
\ *G Fetch and return the two CELLS from memory ADDR and ADDR+sizeof(CELL).
\ ** The cell at the lower address is on the top of the stack.
  sub  # 2 psp
  mov  tos r14
  mov  @r14+ tos
  mov  @r14  0 (psp)
  ret
end-code

CODE C!         \ b addr --
\ *G Store the character CHAR at memory C-ADDR.
  mov .b  @psp  0 (tos)
  add     # 2 psp
  mov     @psp+  tos
  ret
end-code

CODE !          \ n addr --
\ *G Store the CELL quantity N at memory ADDR.
  mov     @psp  0 (tos)
  add     # 2 psp
  mov     @psp+  tos
  ret
end-code

CODE 2!         \ d addr --
\ *G Store the two CELLS x1 and x2 at memory ADDR.
\ ** X2 is stored at ADDR and X1 is stored at ADDR+CELL.
  mov  @psp+  0 (tos)
  mov  @psp+  2 (tos)
  mov  @psp+  tos
  ret
end-code


\ ****************
\ *S String operators
\ ****************

code count      \ addr -- addr+1 len
\ *G Given the address of a counted string in memory this word will
\ ** return the address of the first character and the length in
\ ** characters of the string.
  sub  # 2 psp
  clr  r14
  mov  .b @tos+ r14
  mov  tos  0 (psp)
  mov  r14 tos
  ret
end-code

CODE CMOVE      \ source dest len -- ; copy memory areas
\ *G Copy U bytes of memory forwards from C-ADDR1 to C-ADDR2.
  mov  @psp+ r13		\ r13=dest
  mov  @psp+ r12		\ r12=src
  cmp  # 0 tos
  nz, if,
    begin,
      mov .b  @r12+ 0 (r13)
      inc     r13
      dec     tos
    z, until,
  endif,
  mov  @psp+ tos
  ret
end-code

CODE CMOVE>      \ source dest len -- ; copy memory areas
\ *G As *\fo{CMOVE} but working in the opposite direction,
\ ** copying the last character in the string first.
  mov  @psp+ r13		\ r13=dest
  mov  @psp+ r12		\ r12=src
  cmp  # 0 tos
  nz, if,
    add  tos r12
    add  tos r13
    begin,
      dec     r12
      dec     r13
      mov .b  @r12  0 (r13)
      dec     tos
    z, until,
  endif,
  mov  @psp+ tos
  ret
end-code

\ Fill is used by multi-tasker
: FILL          \ addr len char --
\ *G Fill LEN bytes of memory starting at ADDR with the byte information
\ ** specified as CHAR.
  SWAP >R                               \ save length
  OVER C!                               \ set first character
  DUP 1+  R> 1- CMOVE                   \ copy n-1 chars
;

: (")           \ -- addr ; in-line string follows caller, and skip it
\ *G Return the address of a counted string that is inline after the
\ ** CALLING word, and adjust the CALLING word's return address to
\ ** step over the inline string. See the definition of (.") for
\ ** an example.
  r>					\ return address to caller
  r> dup count + 1+ -2 and >r		\ caller's return address
  swap >r                               \ restore return address
;

: (C")		\ -- c-addr
\ *G The runtime action compiled by C".
  (")
;

: (S")		\ -- c-addr u
\ *G The runtime action compiled by S".
  (") count
;


\ ************************************
\ *S Umbilical versions of defining words
\ ************************************

asmcode
here is-action-of constant
\ *G The runtime action for a *\fo{CONSTANT}.
here is-action-of variable
\ *G The runtime action for a *\fo{VARIABLE}.
  sub  # 2  psp			\ save previous TOS
  mov  tos 0 (psp)
  pop  r14			\ get address of data
  mov  @r14 tos			\ get data
  ret
end-code

asmcode
here is-action-of user
\ *G The runtime action of a *\fo{USER} variable.
  sub  # 2  psp			\ save previous TOS
  mov  tos 0 (psp)
  pop  r14			\ get address of offset
  mov  up tos			\ get USER area pointer
  add  @r14 tos			\ add offset
  ret
end-code

interpreter
: u#		\ "<uservar>" -- offset ; u# <uservar>
\ *G Return the offset of a *\fo{USER} variable in the user area.
  ' >body
;
target

asmcode
here is-action-of value	\ -- n ; default returns the value
\ *G The runtime action of a *\fo{VALUE}.
  sub  # 2  psp			\ save previous TOS
  mov  tos 0 (psp)
  pop  r14              	\ get address of address
  mov  @r14 r14			\ get address
  mov  @r14 tos			\ get contents
  ret
end-code

: CRASH		\ --  ; used as default action of DEFERred word
\ *G The default action of a *\fo{DEFER}ed word. A NOOP.
;

asmcode
here is-action-of DEFER	\ Comp: "<spaces>name" -- ; Run: i*x -- j*x
\ *G The runtime action of a *\fo{DEFER}red word.
  pop  r14			\ followed by address of data in RAM
  mov  @r14 r15			\ get address in RAM
  br   @r15			\ go do it
end-code


\ ******************
\ *S Interrupt handling
\ ******************
\ *P The interrupt handling words are just
\ ** code generators in the cross compiler and have no target
\ ** versions.

decimal


\ ======
\ *> ###
\ ======

