\ Startup entry points and vectors

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

\ ==========
\ *! start2553
\ *T MSP430G2553 start up
\ ==========
\ *P The file *\i{MSP430\Hardware\Launchpad2553\start2553.fth}
\ ** contains Forth start up code and initialisation tables for
\ ** a Launchpad board with an MSP430G2553 CPU.


\ ******************
\ *S Magic addresses
\ ******************
\ *P Some factory calibration data is held in Info Flash in
\ ** TLV (Tag, Length, Value) format. The most interesting of
\ ** this are the DCO and ADC calibration data. For startup, we
\ ** need the DCO calibration data
\ *E $10FF      CALBC1_1MHz
\ ** $10FE      CALDCO_1MHz
\ ** $10FD      CALBC1_8MHz
\ ** $10FC      CALDCO_8MHz
\ ** $10FB      CALBC1_12MHz
\ ** $10FA      CALDCO_12MHz
\ ** $10F9      CALBC1_16MHz
\ ** $10F8      CALDCO_16MHz
\ ** $10F7 08   Size (bytes) of value data
\ ** $10F6 01   Tag

$10FF equ CALBC1_1MHz
$10FE equ CALDCO_1MHz
$10FD equ CALBC1_8MHz
$10FC equ CALDCO_8MHz
$10FB equ CALBC1_12MHz
$10FA equ CALDCO_12MHz
$10F9 equ CALBC1_16MHz
$10F8 equ CALDCO_16MHz

\ *P Port 1 is used as follows:
\ *E P1.0       Green LED, high=on
\ ** P1.1       UART Rx
\ ** P1.2       UART Tx
\ ** P1.3       Button Switch input
\ ** P1.4       --
\ ** P1.5       --
\ ** P1.6       Red LED, high=on
\ ** P1.7       --

\ *****************
\ *S Start of Forth
\ *****************
\ *P This section contains initial values of Forth
\ ** registers and the code to initialise and
\ ** run Forth.
L: CLD1                                 \ cold start word         008
  0 ,

\ #define OFIFG               0x02
\ #define XT2OFF              0x80	\ Enable XT2CLK
#define SELM1               0x80	\ SMCLK Source Select
\ #define SELS                0x08	\ MCLK Source Select 0:DCOCLK

AsmCode
L: ECLD
\ *G The entry point after reset. This code is for a TI Launchpad
\ ** board run from the DCo at 8 MHz. It uses LEDs on P1.6 (red)
\ ** and P1.0 (green) for debugging.
  mov     # $5A80  & WDTCTL		\ stop WDT by setting bit 7

  mov .b  & CALBC1_8MHZ  & BCSCTL1	\ Set DCO TO 8 mHz
  mov .b  & CALDCO_8MHZ  & DCOCTL
  mov .b  # 0 & BCSCTL2			\ force DCO use; belt and braces
  bis .b  # $41  & P1DIR		\ set P1.6 and P1.0 to be outputs
  bis .b  # $41  & P1OUT		\ set both LEDs high (on)
  mov     # 0  sr			\ clear status reg

XT2-speed [if]
\ *G If the equate *\fo{XT2-speed} is set non-zero in the control
\ ** file, the main clock is taken from the high frequency
\ ** oscillator at the frequency specified by *\fo{XT2-speed}.
L: SetupBC
  bic .b  # XT2OFF & BCSCTL1		\ XT2 = HF XTAL
L: SetupOsc
  bic .b  # OFIFG & IFG1		\ Clear OSC fault flag
  mov .w  # 0FFh R15			\ R15 = Delay
L: SetupOsc1
  dec .w  R15				\ Additional delay to ensure start
  jnz     SetupOsc1
  bit .b  # OFIFG & IFG1		\ OSC fault flag set?
  jnz     SetupOsc			\ OSC Fault, clear flag again
  bis .b  # SELM1 SELS + & BCSCTL2	\ MCLK = XT2
L: SetupP5
[then]

  mov .b  # 0 & ie1			\ disable all interrupts
  mov .b  # 0 & ie2

\ Generic Forth startup
  mov     # init-r0 rsp
  mov     # init-s0 psp
  mov     # init-u0 up
  call    cld1
  jmp     ecld
End-Code


\ *************************
\ *S Default Interrupt vectors
\ *************************
\ *P All interrupt vectors are set to point to the *\fo{ECLD}
\ ** entry point above. Later code may modify these settings.
\ *[
ECLD reset_vec	   !
ECLD nmi_vec	   !
ECLD timer1_A0_vec !
ECLD timer1_A1_vec !
ECLD CompA_vec     !
ECLD WDT_vec       !
ECLD timer0_a0_vec !
ECLD timer0_a1_vec !
ECLD UCx0Rx_vec    !
ECLD UCx0Tx_vec    !
ECLD ADC10_vec     !
ECLD P2_vec        !
ECLD P1_vec        !
\ *]


\ ******************************************
\ *S Reset values for user and system variables
\ ******************************************
\ *P The equate *\fo{SP-GUARD} in the control file defines how
\ ** many guard cells are provided at the top of the data
\ ** stack to give some protection against system crashes
\ ** if code underflows the data stack. If *\fo{SP-GUARD} is
\ ** undefined at this point, a default value of 0 is
\ ** defined.

[undefined] sp-guard [if]
0 equ sp-guard	\ -- n
\ *G Default number of data stack guard cells.
[then]

\ *P The section of *\fo{USER} variable initialisation values
\ ** is generated for standalone targets if the equate
\ ** *\fo{UMBILICAL?} is udefined or set to zero.
[undefined] umbilical? [if]
  -1
[else]
  umbilical? 0=
[then]
[if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
\ L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-RP  0 ,				\ RP
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

