\ include.fth - INCLUDE using AIDE as the file server.

((
Copyright (c) 1991..2004, 2005, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====
Permit file nesting.

Change history
==============
20131219 MPE007 Updated for Lite compiler.
20051105 MPE006 Force interpretation on error.
20040505 MPE005 Modified for AIDE 2.500 onwards.
20021004 SFP004 Refactored to use CATCH/THROW
20020917 SFP003 Refactored to reduce size
20000530 SFP002 Converted to v6.1 targets
19990509 SFP001 Rewritten for AIDE/PowerTerm.
                Removed support for USE
First Release: 29/04/91
))

\ ===========
\ *! includeaide
\ *T Compile source code from AIDE
\ ===========
\ *P The file *\i{include.fth} provides support for
\ ** compiling a source file from the AIDE server.

decimal

$01 equ START
$06 equ ACK
$15 equ NAK

$12 equ ^R                                         \ read command code
$17 equ ^W                                         \ write
$15 equ ^U                                         \ $using
$01 equ ^A                                         \ use
$06 equ ^F                                         \ $from
$05 equ ^E                                         \ error

Synonym Ser-Key  key
Synonym Ser-Key? key?
Synonym Ser-Emit emit

variable disk-error

: end-load      \ -- ; switch back to keyboard input
\ *G This word is automatically performed at the end of a download
\ ** to tidy up the comms.
  xon/xoff off                  	\ turn off handshaking
  echoing on                    	\ turn echoing back on
;

internal

: WAIT-ACK  (  -- ; wait for ACK character at the end of part of a transfer  )
  begin
    ser-key                             \ read a char from serial line
    ack =                               \ until it is an ACK
  until
;

: WAIT-ACK/NACK  (  --  flag ; wait for NACk or ACK, return true for NACK  )
  0
  begin
    drop                                \ drop old flag
    ser-key dup nak = tuck              \ -- t/f char t/f
    swap ack =  or                      \ -- t/f t/f'
  until
;

: SEND-BLOCK#  (  n  -- ; send block number to server as two bytes  )
  0 #256 um/mod swap                    \ split into high & low bytes
  ser-emit ser-emit                     \ send block no., low byte first
  wait-ack                              \ wait for ack
;

: SYNCH-TO-HOST  (  -- ; sync host to us  )
  begin                                 \ sync.
    ser-key                             \ get a char
    start =                             \ receive bytes until start code
  until
  begin
    ser-key?                            \ while there are keys still
  while
    ser-key                             \ read them
    drop                                \ and drop them
  repeat
  ack ser-emit                          \ acknowledge as end of synch.
;

: SEND$  (  $addr ^char -- ; send a counted string and trigger to host )
  ser-emit                              \ send code
  synch-to-host                         \ synchronise
  dup 1+ swap                           \ addr+1 as base for reading
  c@ dup send-block#                    \ send length of string
  0                                     \ length - 0
  do
    dup i + c@                          \ get char at i+$addr
    ser-emit                            \ send char
  loop
  drop                                  \ drop spare address
  ack ser-emit                          \ send ack to complete transfer
  wait-ack/nack disk-error !            \ get status response
;

: file-error    \ n --
\ *G Handle an error when a file is being *\fo{INCLUDE}d.
  ^e ser-emit                           \ trigger host error
  CR ." Error # " .  SPACE here count type CR   \ show error
\  CR ." Error # " .  SPACE here $. CR   \ show error
  end-load                              \ clean up
;

: $include	\ $addr -- ; compile host file, counted string
\ *G Given a counted string representing a file name, compile
\ ** the file from AIDE.
  ^f send$                      	\ send file name
  -1 send-block#                   	\ send last-p# - legacy
  1 send-block#                   	\ send first-p# - legacy
  synch-to-host                 	\ synchronise
  echoing off                   	\ not echoing
  xon/xoff on                   	\ initiate handshaking
[ blocks? ] [if]
  blk off
[then]
  0 <source> !  [compile] [		\ set up
  begin
    query				\   get user input
    ['] interpret catch ?dup if		\   interpret line
      file-error			\    display error
      s0 @ sp!				\    clean data stack
      [compile] [			\    force interpretation ; MPE006
    then
    echoing @
  until                                 \   do next line
;

external

: include       \ "<filename>" -- ; load file from host
\ *G Compile a file across the serial line from the AIDE file server.
\ ** Use in the form:
\ *C   include <filename>
\ *P The filename extension must be supplied.
  bl word                               \ get file name
  dup count upper                       \ convert to upper case
  $include				\ compile from file
;


\ ======
\ *> ###
\ ======

decimal

