\ liteLP2553sa.ctl - MSP430 control file for LaunchPad2553 standalone Forth

((
Copyright (c) 2002, 2004, 2007, 2012, 2013, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140125 SFP005 Changed Flash compilation default.
20131218 SFP004 Updated for renamed files and AFTERWARDS.
20120823 SFP003 Converted for Launchpad with MSP430G2553 at 9600 baud.
20070818 SFP002 Changed baud rate to 38400.
20040105 RGB001 Equates added for the various MSP430 clocks.
                Equate 13X_OR_14X? added to specify when a 13x or
		14x device is in use.
))


only forth definitions decimal

Reals                                   \ turn on floating point


\ ******************************
\ Define the default directories
\ ******************************

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ."			setmacro CpuDir		\ CPU specific files
c" ..\Common"		setmacro CommonDir	\ common high level files
c" ..\Examples"		setmacro ExampleDir	\ example files


\ ***********************************************
\ Now turn on the cross compiler and configure it
\ ***********************************************

CROSS-COMPILE
  rommed				\ split ROM/RAM environment
  no-log                		\ turn off symbol log
  align-even
\  -fast-do
  +FlashCompile				\ target compiles to Flash
  smaller
  +xrefs                		\ turn on XREF facility
  interactive           		\ finish with compiler running
\  -download-prompts

only forth definitions                  \ initial search order


\ ***********************************************************************
\ Define target configuration here. These flags define whether certain
\ files are loaded, and whether certain initialisation words are included
\ in COLD.
\ ***********************************************************************

0  Equ False
-1 Equ True

decimal

\ Does this CPU version have the multiplier?
0 equ multiplier?		\ true if multiplier present

0 equ xtal-speed		\ no main cystal
0 equ LFXT1-speed		\ speed of main low/high frequency crystal or
                                \ resonator ; RGB001
12 kHz equ ACLK-speed		\ alternative (low power) clock

0 equ XT2-speed			\ speed of alternative high speed crystal
                                \ Set to 0 if XT2 not fitted

8 MHz equ DCOCLK-speed		\ desired DCO speed
DCOCLK-speed equ MCLK-speed	\ speed of main internal clock
DCOCLK-speed equ SMCLK-speed	\ speed of subsystem internal clock

DCOCLK-speed equ system-speed	\ CPU clock speed

1 equ useUSCIA0?		\ for UART A0
#9600 equ console0-speed	\ Serial port speed in BPS.
0 equ useUSCIA1?		\ for UART A1
#9600 equ console1-speed	\ Serial port speed in BPS.

0 equ console-port		\ 0..9 for UARTs, 10=VUART

0 equ ColdChain?		\ nz to use cold chain mechanism
\ Task Control Block
  6 cells equ tcb-size		\   for internal consistency check
  0 equ event-handler?		\   true to include event handler
  0 equ message-handler?	\   true to include message handler
  1 equ semaphores?		\ true to include semaphores
0 equ #irqs                     \ how many "stacked" interrupt levels are needed
                                \ set to non-zero if needed
                                \ stacked means: can interrupt an interrupt
0 equ timebase?			\ true to load TIMEBASE code
0 equ target-locals?            \ true if target needs to compile local variables
0 equ blocks?			\ true if BLOCK support needed
0 equ paged?                    \ true if target is paged
0 equ softfp?                   \ true if target needs floating point
$0000 equ sizeofheap		\ 0=no heap, nz=size of heap
  0 equ heap-diags?		\   true to include diagnostic code
0 equ FullCase?			\ true if target needs to compile NEXTCASE END-CASE ?OF


\ ******************
\ Memory definitions
\ ******************
\ Many of these are CPU specific

  $D400 $ffff cdata section liteLP2553sa	\ code
    all-saved                           \ save whole section with vectors
    sec-base sec-len $FF fill		\ Empty flash contains $FF
  $0200 $027F idata section PROGI	\ initialised data
  $0280 $03FF udata section PROGU	\ uninitialised data

liteLP2553sa PROGI PROGU			\ Select current sections
cdata					\ use Code for HERE

0x1000 equ INFOSTART   		\ 4 segments of 64 bytes
0x1040 equ APPSTART
0x10FF equ INFOEND
0x0200 equ RAMSTART
0x0400 equ RAMEND
0xC000 equ FLASHSTART  		\ start of Main Flash
0xC000 equ APPFLASHSTART  	\ start of application flash
0xD400 equ APPFLASHEND  	\ end of application flash
0x1400 equ /APPFLASH  		\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up


\ ******************************************************
\ Define the stacks and user area and the memory layout
\ ******************************************************

\ MAIN task
$0A0 EQU UP-size		\ user area len
$040 EQU RP-Size                \ return stack length
$040 EQU SP-Size                \ parameter stack length
$050 EQU TIB-LEN                \ terminal i/p buffer length
\ define the number of cells of guard space at the top of the data stack
#1 equ sp-guard

0				\ initial offset of USER area
  Equ Task-u0
UP-size SP-size +		\ initial offset of data stack
  Equ Task-s0
UP-size SP-size + RP-size +	\ initial offset of return stack
  Equ Task-r0
UP-size SP-size + RP-size +	\ system area size for each task
  Equ task-size

task-size reserve equ INIT-T0		\ base of main task user area
  INIT-T0 Task-U0 + EQU INIT-U0		\ base of USER area
  INIT-T0 Task-R0 + EQU INIT-R0		\ top of return stack
  INIT-T0 task-S0 + EQU INIT-S0		\ top of data stack

TIB-LEN  reserve equ INIT-TIB		\ base of TIB


\ *****************
\ Load kernel forth
\ *****************

  include %CpuDir%\sfr430		\ generic MSP430 equates
  include %CpuDir%\sfr2xx		\ MSP430x2xx I/O register definitions
  include %CpuDir%\vec2553		\ 2553 interrupt vectors
l: crcslot
  0 ,
l: crcstart
  include %HwDir%\start2553		\ system set up
  include %CpuDir%\code430lite		\ main code defs.
  include %CpuDir%\delays		\ time delays
  include %CpuDir%\Drivers\LedTickLP2553 \ ticker
  include %CpuDir%\kernel72lite		\ high level kernel
  include %CpuDir%\drivers\serUSCIp	\ polled serial I/O
\  include %CpuDir%\value		\ method system for VALUEs
\  include %CpuDir%\local		\ local variable runtime
  include %CpuDir%\dump			\ nucleus extensions

#irqs [if]
  include %CpuDir%\intrpt430		\ interrupts
[then]

softfp? [if]				\ Software Floating Point
  include %CommonDIR%\sfp16hi		\ high level primitives
  include %CommonDIR%\sfp16com		\ common code
[then]

  include %CpuDir%\include		\ include through AIDE.


\ ***************
\ Sign on message
\ ***************

internal
: .cpu          \ -- ; display CPU type
  ." MPE MSP430 Forth Lite v1.2"
;
external


\ ***************************************
\ Add Application files to be loaded here
\ ***************************************

\ include %ExampleDIR%\primes		\ a simple benchmark
\  include test430.fth			\ MSP430 compiler test code


\ *************
\ Final tidy up
\ ************

libraries	\ resolve required forward references
  include %CpuDir%\lib430
  include %CpuDir%\library
end-libs

RAMEND constant RP-END	\ end of available RAM

make-turnkey COLD

\ Add a kernel checksum
crcstart here crcslot crc16 checksum
/DefStart 62 > [if]
  .( DEFSTART area too big ) abort
[then]

\  +DebugFET
  #3000 set-vcc				\ set default VCC to 3000mv
  afterwards USB FETdefine		\ USB FET
  afterwards FETdownload		\ will be done after FINIS

cr
cr ." To download through the FET to the target, type:"
cr ."   FETdownload"
cr ." You only need to download the liteLP2553sa section."
cr ." Answer Y to erase the Flash."
cr ." When the device is programmed, start the target."
cr ."  430run"
cr

FINIS

