\ liteLP2553uf.ctl - Umbilical Forth control file for TI Launchpad

((
Copyright (c) 2002, 2007, 2009, 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140623 SFP004 Updated for FET reliability.
20131218 SFP003 Updated for renamed files and AFTERWARDS.
20120905 SFP002 Reworked for Launchpad with MSP430G2553
20031001 RGB001 Reworked for flexibility
))


Only Forth Definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ."			setmacro CpuDir		\ CPU specific files
c" ..\Examples"		setmacro ExampleDir	\ example files


\ ****************************************************************
\ Add any files that extend the host, for example assembler macros
\ ****************************************************************


\ **********************************************************
\ If you want the compiler log to go to a file, name it here
\ **********************************************************

\ File: PROG.log


\ **************************************
\ Start and set up the cross compilation
\ **************************************

CROSS-COMPILE

  no-heads                      \ No heads on target
  No-log			\ No output log
  Rommed                        \ Split ROM/RAM system
  Align-Even			\ 16 bit aligned
  +xrefs                	\ turn on XREF facility
\  smaller
  faster

Only Forth Definitions          \ default search order


\ ********************
\ Target Configuration
\ ********************

0  Equ False
-1 Equ True

0 equ xtal-speed		\ no main cystal
0 equ LFXT1-speed		\ speed of main low/high frequency crystal or
                                \ resonator ; RGB001
12 kHz equ ACLK-speed		\ alternative (low power) clock

0 equ XT2-speed			\ speed of alternative high speed crystal
                                \ Set to 0 if XT2 not fitted

8 MHz equ DCOCLK-speed		\ desired DCO speed
DCOCLK-speed equ MCLK-speed	\ speed of main internal clock
DCOCLK-speed equ SMCLK-speed	\ speed of subsystem internal clock

DCOCLK-speed equ system-speed	\ CPU clock speed

#10 equ console-port		\ 0..9 for UARTS, 10 for VUART

\ Does this CPU version have the multiplier?
0 equ multiplier?		\ true if multiplier present

\ Do we need interactive debugging over the XTL?
1 equ umbilical?		\ true if debugging over XTL
  0 equ SPI?			\   true if XTL over SPI port

\ Does the initialised data area actually need to be initialised?
0 equ init-idata?		\ Interrupt handler may need it

\ Task Control block
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   0 equ semaphores?		\ true to include semaphores
0 equ target-locals?            \ true if target needs local variables
0 equ paged?                    \ true if target is paged
0 equ softfp?                   \ true if target needs floating point
0 equ sizeofheap                \ 0=no heap, nz=size of heap
  0 equ heap-diags?		\   true to include diagnostic code
0 equ #irqs                     \ how many "stacked" interrupt levels are needed
                                \ set to non-zero if needed
                                \ stacked means: can interrupt an interrupt

\ **********
\ Memory map
\ **********

  $C000 $ffff cdata section liteLP2553uf	\ code
    all-saved                           \ save whole section with vectors
    sec-base sec-len $FF fill		\ Empty flash contains $FF
  $0200 $02FF idata section PROGI	\ initialised data
    sec-top 1+ equ UNUSED-TOP		\ top of memory for UNUSED
  $0300 $03FF udata section PROGU	\ uninitialised data

liteLP2553uf PROGI PROGU			\ Select current sections
cdata					\ use Code for HERE


\ *********************
\ Define RAM allocation
\ *********************

\ These equates calculate how much RAM is needed for the USER area,
\ Return stack and data stack of each task, plus an area used
\ by Forth interrupts that share a common USER (and optionally)
\ common stacks. This can be done to reduce the overall amount
\ of RAM needed for interrupt stacks. Since interrupts can occur
\ in any task, switching stacks for high level interrupts will
\ reduce the total RAM requirements.

\ define user area and stack sizes
$020 EQU UP-Size			\ user area len
$070 EQU RP-Size                        \ return stack length
$070 EQU SP-Size                        \ parameter stack length
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard

$38 Equ max-depth			\ max depth for UF data stack

0					\ initial offset of USER area
  Equ Task-u0
UP-size SP-size +			\ initial offset of data stack
  Equ Task-s0
UP-size SP-size + RP-size +		\ initial offset of return stack
  Equ Task-r0
UP-size SP-size + RP-size +		\ system area size for each task
  Equ task-size

$20 EQU IRQ-UP-Size			\ for interrupt service routines
$070 EQU IRQ-RP-size
$070 EQU IRQ-SP-size

IRQ-UP-size IRQ-SP-size + IRQ-RP-size +	\ system area size for each ISR
  Equ per-irq

\ reserve space for the main task and the high level ISRs
\ MAIN task
\ length            name
task-size reserve equ INIT-T0		\ base of main task user area
  INIT-T0 Task-U0 + EQU INIT-U0		\ base of USER area
  INIT-T0 Task-R0 + EQU INIT-R0		\ top of return stack
  INIT-T0 task-S0 + EQU INIT-S0		\ top of data stack

((
\ for high level interrupt handlers
0        reserve equ TOP-IRQ		\ top of IRQ stacks
#irqs 1+ per-irq *			\ need at least one
         reserve equ BOT-IRQ		\ bottom of IRQ stacks
))

\ ************
\ Kernel files
\ ************

include %CpuDir%\sfr430			\ generic MSP430 equates
include %CpuDir%\sfr2xx			\ I/O register definitions
include %CpuDir%\vec2553		\ vector definitions

include %CpuDir%\start2553		\ system set up
Include %CpuDir%\min430			\ code definitions
Include %CpuDir%\Drivers\VUART		\ Virtual UART driver
Include %CpuDir%\UFKernel		\ Umbilical Forth high level kernel
Umbilical? [if]
  Synonym wait-byte  keyv		\ Say which XTL drivers to use
  Synonym send-byte  emitv
  Synonym Wait-Byte? key?v
  Synonym Init-XTL   InitVuart
  Include %CpuDir%\messages		\ Equates only
  Include %CpuDir%\Targend		\ Message driver
[Endif]


\ *************
\ End of Kernel
\ *************

: .CPU          \ -- ; sign on
  ." MPE MSP430 "
  [ Umbilical? ] [if] ." Umbilical " [else] ." Production " [then]
  ." Forth"
;


\ **************************
\ Load application code here
\ **************************

   include %CpuDir%/Drivers/gpio2553	\ simple I/O drivers
   include %CpuDir%/Drivers/TickLP2553
   include %AppDir%/project0uf.fth	\ you can use a macro for APPDIR
\  get tests


\ *******************
\ Load libraries here
\ *******************

libraries
  include %CpuDir%\lib430
  include %CpuDir%\UFlib
end-libs


\ **************************************
\ Define what happens at target power up
\ **************************************

((
: test-UART	\ -- char
  ipvec @ >r  opvec @ >r		\ currently points to VUART
  init-ser				\ initialise UART
  console0 dup ipvec !  opvec !		\ switch o/p to UART
  ." Is the UART working? " key
  r> opvec !  r> ipvec !
;
))

Umbilical? [if]
  make-turnkey run-umbilical            \ in TARGEND.FTH
  cr ." The code in this file defaults to a USB FET."
  #3000 set-vcc				\ set default VCC to 3000mv
  USB FETdefine				\ USB FET
  +DebugFET
  cr ." You only need to download the LITELP2553UF section."
  cr ." Answer Y to erase the Flash."
  cr
  cr ." When you get the T:OK prompt, type .CPU to run"
  cr ." the word .CPU from this file."
  cr
\  cr ." Press any key to continue"  key drop
\  cr
  +FET-umbilical			\ Assume CPU is running
  afterwards +FET-umbilical
  afterwards 430reopen
  afterwards 430go
  umbilical-forth                       \ start interactive session
[else]
: test-start    \ --
  initVuart				\ initialise the VUART for our use
  console ipvec !  console opvec !	\ I/O goes to VUART
  c" This is the MSP430 Chip speaking!"
  count 0
  do  dup c@ emit 1+  loop  \  drop
  begin
    key emit
  again
;

  Make-turnkey test-start		\ application start up word
[then]

finis

