\ MSP430 Standard special function registers and equates

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))



\ ************************
\ Standard bit definitions
\ ************************

\ bit masks
#define BIT0    0x0001
#define BIT1    0x0002
#define BIT2    0x0004
#define BIT3    0x0008
#define BIT4    0x0010
#define	BIT5    0x0020
#define BIT6    0x0040
#define BIT7    0x0080
#define BIT8    0x0100
#define BIT9    0x0200
#define BIT10   0x0400
#define BIT11   0x0800
#define BIT12   0x1000
#define BIT13   0x2000
#define BIT14   0x4000
#define BIT15   0x8000
#define BITA    0x0400
#define BITB    0x0800
#define BITC    0x1000
#define BITD    0x2000
#define BITE    0x4000
#define BITF    0x8000


\  Status Register Bits
#define _CARRY     0x0001
#define _ZERO      0x0002
#define _NEGATIVE  0x0004
#define _GIE       0x0008
#define _CPUOFF    0x0010
#define _OSCOFF    0x0020
#define _SCG0      0x0040
#define _SCG1      0x0080
#define _OVERFLOW  0x0100


\ **************************
\ Special function registers
\ **************************

\ the standard ones
#define IE1  0x0000
#define IE2  0x0001
#define IFG1 0x0002
#define IFG2 0x0003
#define ME1  0x0004
#define ME2  0x0005

\ Watchdog/Timer register frame
#define WDTCTL 0x0120

\  HARDWARE MULTIPLIER frame
#define MPY    0x0130
#define MPYS   0x0132
#define MAC    0x0134
#define MACS   0x0136
#define OP2    0x0138
#define RESLO  0x013A
#define RESHI  0x013C
#define SUMEXT 0x013E

\  Digital I/O frame
#define P1IN  0x0020
#define P1OUT 0x0021
#define P1DIR 0x0022
#define P1IFG 0x0023
#define P1IES 0x0024
#define P1IE  0x0025
#define P1SEL 0x0026

#define P2IN  0x0028
#define P2OUT 0x0029
#define P2DIR 0x002A
#define P2IFG 0x002B
#define P2IES 0x002C
#define P2IE  0x002D
#define P2SEL 0x002E

\ Interrupts
$FFFE equ reset_vec
$FFFC equ nmi_vec

\
\ Core peripheral module bit definitions.
\
\ System Clock Control
\ BCSCTL1 register
#define RSEL.0    0x0001
#define RSEL.1    0x0002
#define RSEL.2    0x0004
#define XT5V      0x0008
#define DIVA.0    0x0010
#define DIVA.1    0x0020
#define XTS       0x0040
#define XT2OFF    0x0080

\ BCSCTL2 register
#define DCOR      0x0001
#define DIVS.0    0x0002
#define DIVS.1    0x0004
#define SELS      0x0008
#define DIVM.0    0x0010
#define DIVM.1    0x0020
#define SELM.0    0x0040
#define SELM.1    0x0080

\ CACTL1 register
#define CAIFG     0x0001
#define CAIE      0x0002
#define CAIES     0x0004
#define CAON      0x0008
#define CAREF0    0x0010
#define CAREF1    0x0020
#define CARSEL    0x0040
#define CAEX      0x0080

\ CACTL2 register
#define CAOUT     0x0001
#define CAF       0x0002
#define CA0       0x0004
#define CA1       0x0008
#define CACTL2.4  0x0010
#define CACTL2.5  0x0020
#define CACTL2.6  0x0040
#define CACTL2.7  0x0080

\ DCOCTL register
#define MOD.0     0x0001
#define MOD.1     0x0002
#define MOD.2     0x0004
#define MOD.3     0x0008
#define MOD.4     0x0010
#define DCO.0     0x0020
#define DCO.1     0x0040
#define DCO.2     0x0080


\ Timer A module
\ TACTL register
#define TAIFG     0x0001
#define TAIE      0x0002
#define TACLR     0x0004
#define MC0       0x0010
#define MC1       0x0020
#define ID0       0x0040
#define ID1       0x0080
#define TASSEL0   0x0100
#define TASSEL1   0x0200
#define TASSEL2   0x0400

\ CCTL0-2 register
#define CCIFG     0x0001
#define COV       0x0002
#define OUT       0x0004
#define CCI       0x0008
#define CCIE      0x0010
#define OUTMOD0   0x0020
#define OUTMOD1   0x0040
#define OUTMOD2   0x0080
#define CAP       0x0100
#define SCCI      0x0400
#define SCS       0x0800
#define CCIS0     0x1000
#define CCIS1     0x2000
#define CM0       0x4000
#define CM1       0x8000

\ FLASH programming
\ FCTL1 register
#define FCT_PSWD  0xA500

#define ERASE     0x0002
#define MERAS     0x0004
#define WRT       0x0040
#define SEGWRT    0x0080

\ FCTL2 register
#define FN0       0x0001
#define FN1       0x0002
#define FN2       0x0004
#define FN3       0x0008
#define FN4       0x0010
#define FN5       0x0020
#define SSEL0     0x0040
#define SSEL1     0x0080

\ FCTL3 register
#define BUSY      0x0001
#define KEYV      0x0002
#define ACCVIFG   0x0004
#define WAIT      0x0008
#define FCTLOCK   0x0010
#define EMEX      0x0020

\ Watchdog module
\ WDTCTL
#define WDT_PSWD  0x5A00

#define IS0       0x0001
#define IS1       0x0002
#define SSEL      0x0004
#define CNTCL     0x0008
#define TMSEL     0x0010
#define NMI       0x0020
#define NMIES     0x0040
#define WDTHOLD   0x0080
